@@ -11,21 +11,25 @@ PETSC_EXTERN PetscErrorCode KSPInitializePackage(void);
 PETSC_EXTERN PetscErrorCode KSPFinalizePackage(void);
 
 /*S
-   KSP - Abstract PETSc object that manages all Krylov methods. This is the object that manages the
-         linear solves in PETSc (even those such as direct factorization-based solvers that do no use Krylov accelerators).
+   KSP - Abstract PETSc object that manages the linear solves in PETSc (even those such as direct factorization-based solvers that
+         do not use Krylov accelerators).
 
    Level: beginner
 
-   Note:
+   Notes:
    When a direct solver is used, but no Krylov solver is used, the `KSP` object is still used but with a
    `KSPType` of `KSPPREONLY` (or equivalently `KSPNONE`), meaning that only application of the preconditioner is used as the linear solver.
 
+   Use `KSPSetType()` or the options database key `-ksp_type` to set the specific Krylov solver algorithm to use with a given `KSP` object
+
+   The `PC` object is used to control preconditioners in PETSc.
+
 .seealso: [](doc_linsolve), [](ch_ksp), `KSPCreate()`, `KSPSetType()`, `KSPType`, `SNES`, `TS`, `PC`, `KSP`, `KSPDestroy()`, `KSPCG`, `KSPGMRES`
 S*/
 typedef struct _p_KSP *KSP;
 
 /*J
-   KSPType - String with the name of a PETSc Krylov method.
+   KSPType - String with the name of a PETSc Krylov method. These are all the Krylov solvers that PETSc provides.
 
    Level: beginner
 
@@ -160,13 +164,13 @@ PETSC_EXTERN PetscErrorCode KSPSetNestLevel(KSP, PetscInt);
 PETSC_EXTERN PetscErrorCode KSPGetNestLevel(KSP, PetscInt *);
 
 PETSC_EXTERN PetscErrorCode KSPMonitor(KSP, PetscInt, PetscReal);
-PETSC_EXTERN PetscErrorCode KSPMonitorSet(KSP, PetscErrorCode (*)(KSP, PetscInt, PetscReal, void *), void *, PetscErrorCode (*)(void **));
+PETSC_EXTERN PetscErrorCode KSPMonitorSet(KSP, PetscErrorCode (*)(KSP, PetscInt, PetscReal, void *), void *, PetscCtxDestroyFn *);
 PETSC_EXTERN PetscErrorCode KSPMonitorCancel(KSP);
 PETSC_EXTERN PetscErrorCode KSPGetMonitorContext(KSP, void *);
 PETSC_EXTERN PetscErrorCode KSPGetResidualHistory(KSP, const PetscReal *[], PetscInt *);
-PETSC_EXTERN PetscErrorCode KSPSetResidualHistory(KSP, PetscReal[], PetscInt, PetscBool);
+PETSC_EXTERN PetscErrorCode KSPSetResidualHistory(KSP, PetscReal[], PetscCount, PetscBool);
 PETSC_EXTERN PetscErrorCode KSPGetErrorHistory(KSP, const PetscReal *[], PetscInt *);
-PETSC_EXTERN PetscErrorCode KSPSetErrorHistory(KSP, PetscReal[], PetscInt, PetscBool);
+PETSC_EXTERN PetscErrorCode KSPSetErrorHistory(KSP, PetscReal[], PetscCount, PetscBool);
 
 PETSC_EXTERN PetscErrorCode KSPBuildSolutionDefault(KSP, Vec, Vec *);
 PETSC_EXTERN PetscErrorCode KSPBuildResidualDefault(KSP, Vec, Vec, Vec *);
@@ -178,6 +182,7 @@ PETSC_EXTERN PetscErrorCode PCKSPSetKSP(PC, KSP);
 PETSC_EXTERN PetscErrorCode PCBJacobiGetSubKSP(PC, PetscInt *, PetscInt *, KSP *[]);
 PETSC_EXTERN PetscErrorCode PCASMGetSubKSP(PC, PetscInt *, PetscInt *, KSP *[]);
 PETSC_EXTERN PetscErrorCode PCGASMGetSubKSP(PC, PetscInt *, PetscInt *, KSP *[]);
+PETSC_EXTERN PetscErrorCode PCPatchGetSubKSP(PC, PetscInt *, KSP *[]);
 PETSC_EXTERN PetscErrorCode PCFieldSplitGetSubKSP(PC, PetscInt *, KSP *[]);
 PETSC_EXTERN PetscErrorCode PCFieldSplitSchurGetSubKSP(PC, PetscInt *, KSP *[]);
 PETSC_EXTERN PetscErrorCode PCMGGetSmoother(PC, PetscInt, KSP *);
@@ -455,7 +460,6 @@ PETSC_EXTERN PetscErrorCode KSPSetFromOptions(KSP);
 PETSC_EXTERN PetscErrorCode KSPResetFromOptions(KSP);
 
 PETSC_EXTERN PetscErrorCode KSPMonitorSetFromOptions(KSP, const char[], const char[], void *);
-PETSC_EXTERN PetscErrorCode KSPMonitorLGCreate(MPI_Comm, const char[], const char[], const char[], PetscInt, const char *[], int, int, int, int, PetscDrawLG *);
 PETSC_EXTERN PetscErrorCode KSPMonitorResidual(KSP, PetscInt, PetscReal, PetscViewerAndFormat *);
 PETSC_EXTERN PetscErrorCode KSPMonitorResidualDraw(KSP, PetscInt, PetscReal, PetscViewerAndFormat *);
 PETSC_EXTERN PetscErrorCode KSPMonitorResidualDrawLG(KSP, PetscInt, PetscReal, PetscViewerAndFormat *);
@@ -518,7 +522,7 @@ PETSC_EXTERN PetscErrorCode KSPView(KSP, PetscViewer);
 PETSC_EXTERN PetscErrorCode KSPLoad(KSP, PetscViewer);
 PETSC_EXTERN PetscErrorCode KSPViewFromOptions(KSP, PetscObject, const char[]);
 PETSC_EXTERN PetscErrorCode KSPConvergedReasonView(KSP, PetscViewer);
-PETSC_EXTERN PetscErrorCode KSPConvergedReasonViewSet(KSP, PetscErrorCode (*)(KSP, void *), void *vctx, PetscErrorCode (*)(void **));
+PETSC_EXTERN PetscErrorCode KSPConvergedReasonViewSet(KSP, PetscErrorCode (*)(KSP, void *), void *, PetscCtxDestroyFn *);
 PETSC_EXTERN PetscErrorCode KSPConvergedReasonViewFromOptions(KSP);
 PETSC_EXTERN PetscErrorCode KSPConvergedReasonViewCancel(KSP);
 PETSC_EXTERN PetscErrorCode KSPConvergedRateView(KSP, PetscViewer);
@@ -545,6 +549,7 @@ PETSC_EXTERN PetscErrorCode KSPLSQRMonitorResidualDrawLGCreate(PetscViewer, Pets
 PETSC_EXTERN PetscErrorCode PCRedundantGetKSP(PC, KSP *);
 PETSC_EXTERN PetscErrorCode PCRedistributeGetKSP(PC, KSP *);
 PETSC_EXTERN PetscErrorCode PCTelescopeGetKSP(PC, KSP *);
+PETSC_EXTERN PetscErrorCode PCMPIGetKSP(PC, KSP *);
 
 /*E
    KSPNormType - Norm calculated by the `KSP` and passed in the Krylov convergence
@@ -618,7 +623,7 @@ M*/
 
 PETSC_EXTERN PetscErrorCode KSPSetNormType(KSP, KSPNormType);
 PETSC_EXTERN PetscErrorCode KSPGetNormType(KSP, KSPNormType *);
-PETSC_EXTERN PetscErrorCode KSPSetSupportedNorm(KSP ksp, KSPNormType, PCSide, PetscInt);
+PETSC_EXTERN PetscErrorCode KSPSetSupportedNorm(KSP, KSPNormType, PCSide, PetscInt);
 PETSC_EXTERN PetscErrorCode KSPSetCheckNormIteration(KSP, PetscInt);
 PETSC_EXTERN PetscErrorCode KSPSetLagNorm(KSP, PetscBool);
 
@@ -641,7 +646,7 @@ PETSC_EXTERN PetscErrorCode KSPSetLagNorm(KSP, PetscBool);
 .  `KSP_DIVERGED_ITS`              - requested number of iterations
 .  `KSP_DIVERGED_DTOL`             - large increase in the residual norm
 .  `KSP_DIVERGED_BREAKDOWN`        - breakdown in the Krylov method
-.  `KSP_DIVERGED_BREAKDOWN_BICG`   - breakdown in the `KSPBGCS` Krylov method
+.  `KSP_DIVERGED_BREAKDOWN_BICG`   - breakdown in the `KSPBCGS` Krylov method
 .  `KSP_DIVERGED_NONSYMMETRIC`     - the operator or preonditioner was not symmetric for a `KSPType` that requires symmetry
 .  `KSP_DIVERGED_INDEFINITE_PC`    - the preconditioner was indefinite for a `KSPType` that requires it be definite
 .  `KSP_DIVERGED_NANORINF`         - a not a number of infinity was detected in a vector during the computation
@@ -754,7 +759,7 @@ M*/
    KSP_DIVERGED_BREAKDOWN - A breakdown in the Krylov method was detected so the
    method could not continue to enlarge the Krylov space. Could be due to a singular matrix or
    preconditioner. In `KSPHPDDM`, this is also returned when some search directions within a block
-   are colinear.
+   are collinear.
 
    Level: beginner
 
@@ -782,7 +787,7 @@ M*/
 /*MC
    KSP_DIVERGED_INDEFINITE_PC - It appears the preconditioner is indefinite (has both
    positive and negative eigenvalues) and this Krylov method (`KSPCG`) requires it to
-   be positive definite
+   be symmetric positive definite (SPD).
 
    Level: beginner
 
@@ -907,7 +912,6 @@ PETSC_EXTERN PetscErrorCode PCSetPreSolve(PC, PetscErrorCode (*)(PC, KSP));
 PETSC_EXTERN PetscErrorCode PCPreSolve(PC, KSP);
 PETSC_EXTERN PetscErrorCode PCPostSolve(PC, KSP);
 
-#include <petscdrawtypes.h>
 PETSC_EXTERN PetscErrorCode KSPMonitorLGRange(KSP, PetscInt, PetscReal, void *);
 
 PETSC_EXTERN PetscErrorCode PCShellSetPreSolve(PC, PetscErrorCode (*)(PC, KSP, Vec, Vec));
@@ -918,6 +922,10 @@ PETSC_EXTERN PetscErrorCode PCShellSetPostSolve(PC, PetscErrorCode (*)(PC, KSP,
 
    Level: intermediate
 
+   Note:
+   These methods generate initial guesses based on a series of previous, related, linear solves. For example,
+   in implicit time-stepping with `TS`.
+
 .seealso: [](ch_ksp), `KSPCreate()`, `KSPGuessSetType()`, `KSPGuessType`
 S*/
 typedef struct _p_KSPGuess *KSPGuess;
@@ -927,7 +935,7 @@ typedef struct _p_KSPGuess *KSPGuess;
 
    Values:
  + `KSPGUESSFISCHER` - methodology developed by Paul Fischer
- - `KSPGUESSPOD`     - methodology based on proper orthogonal decomposition
+ - `KSPGUESSPOD`     - methodology based on proper orthogonal decomposition (POD)
 
    Level: intermediate
 
@@ -961,7 +969,7 @@ PETSC_EXTERN PetscErrorCode KSPGetInitialGuessKnoll(KSP, PetscBool *);
     Level: intermediate
 
 .seealso: `MatSchurComplementGetAinvType()`, `MatSchurComplementSetAinvType()`, `MatSchurComplementGetPmat()`, `MatGetSchurComplement()`,
-          `MatCreateSchurComplementPmat()`
+          `MatCreateSchurComplementPmat()`, `MatCreateSchurComplement()`
 E*/
 typedef enum {
   MAT_SCHUR_COMPLEMENT_AINV_DIAG,
@@ -1042,7 +1050,7 @@ PETSC_EXTERN const char *const MatLMVMSymBroydenScaleTypes[];
 PETSC_EXTERN PetscErrorCode MatLMVMSymBroydenSetScaleType(Mat, MatLMVMSymBroydenScaleType);
 
 /*E
-  MatLMVMDenseType - Memory storage strategy for dense variants `MATLMVM`.
+  MatLMVMDenseType - Memory storage strategy for dense variants of `MATLMVM`.
 
   Values:
 + `MAT_LMVM_DENSE_REORDER` - reorders memory to minimize kernel launch
@@ -1130,3 +1138,5 @@ PETSC_EXTERN PetscErrorCode DMCheckInterpolator(DM, Mat, Mat, Mat, PetscReal);
 
 PETSC_EXTERN PetscErrorCode PCBJKOKKOSSetKSP(PC, KSP);
 PETSC_EXTERN PetscErrorCode PCBJKOKKOSGetKSP(PC, KSP *);
+
+PETSC_EXTERN PetscErrorCode DMCopyDMKSP(DM, DM);
