@@ -1,4 +1,6 @@
 #include <petsc/private/petscfeimpl.h> /*I "petscfe.h" I*/
+#include <petsc/private/loghandlerimpl.h>
+#include <../src/sys/logging/handler/impls/default/logdefault.h>
 
 #if defined(PETSC_HAVE_OPENCL)
 
@@ -12,7 +14,7 @@ static PetscErrorCode PetscFEDestroy_OpenCL(PetscFE fem)
   PetscCall(clReleaseContext(ocl->ctx_id));
   ocl->ctx_id = 0;
   PetscCall(PetscFree(ocl));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
   #define PetscCallSTR(err) \
@@ -48,7 +50,7 @@ static PetscErrorCode PetscFEOpenCLGenerateIntegrationCode(PetscFE fem, char **s
   char            *string_tail   = *string_buffer;
   char            *end_of_buffer = *string_buffer + buffer_length;
   char             float_str[] = "float", double_str[] = "double";
-  char            *numeric_str    = &(float_str[0]);
+  char            *numeric_str    = &float_str[0];
   PetscInt         op             = ocl->op;
   PetscBool        useField       = PETSC_FALSE;
   PetscBool        useFieldDer    = PETSC_TRUE;
@@ -78,7 +80,7 @@ static PetscErrorCode PetscFEOpenCLGenerateIntegrationCode(PetscFE fem, char **s
                                     "#  pragma OPENCL EXTENSION cl_amd_fp64: enable\n"
                                     "#endif\n",
                                     &count));
-    numeric_str = &(double_str[0]);
+    numeric_str = &double_str[0];
   }
   /* Kernel API */
   PetscCallSTR(PetscSNPrintfCount(string_tail, end_of_buffer - string_tail,
@@ -430,7 +432,7 @@ static PetscErrorCode PetscFEOpenCLGenerateIntegrationCode(PetscFE fem, char **s
                                   "  return;\n"
                                   "}\n",
                                   &count));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 static PetscErrorCode PetscFEOpenCLGetIntegrationKernel(PetscFE fem, PetscBool useAux, cl_program *ocl_prog, cl_kernel *ocl_kernel)
@@ -460,7 +462,7 @@ static PetscErrorCode PetscFEOpenCLGetIntegrationKernel(PetscFE fem, PetscBool u
   }
   PetscCall(PetscFree(buffer));
   *ocl_kernel = clCreateKernel(*ocl_prog, "integrateElementQuadrature", &err);
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 static PetscErrorCode PetscFEOpenCLCalculateGrid(PetscFE fem, PetscInt N, PetscInt blockSize, size_t *x, size_t *y, size_t *z)
@@ -476,25 +478,25 @@ static PetscErrorCode PetscFEOpenCLCalculateGrid(PetscFE fem, PetscInt N, PetscI
     if (*x * *y == (size_t)Nblocks) break;
   }
   PetscCheck(*x * *y == (size_t)Nblocks, PETSC_COMM_SELF, PETSC_ERR_ARG_SIZ, "Could not find partition for %" PetscInt_FMT " with block size %" PetscInt_FMT, N, blockSize);
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 static PetscErrorCode PetscFEOpenCLLogResidual(PetscFE fem, PetscLogDouble time, PetscLogDouble flops)
 {
-  PetscFE_OpenCL   *ocl = (PetscFE_OpenCL *)fem->data;
-  PetscStageLog     stageLog;
-  PetscEventPerfLog eventLog = NULL;
-  int               stage;
+  PetscLogHandler h;
 
   PetscFunctionBegin;
-  PetscCall(PetscLogGetStageLog(&stageLog));
-  PetscCall(PetscStageLogGetCurrent(stageLog, &stage));
-  PetscCall(PetscStageLogGetEventPerfLog(stageLog, stage, &eventLog));
-  /* Log performance info */
-  eventLog->eventInfo[ocl->residualEvent].count++;
-  eventLog->eventInfo[ocl->residualEvent].time += time;
-  eventLog->eventInfo[ocl->residualEvent].flops += flops;
-  PetscFunctionReturn(0);
+  PetscCall(PetscLogGetDefaultHandler(&h));
+  if (h) {
+    PetscEventPerfInfo *eventInfo;
+    PetscFE_OpenCL     *ocl = (PetscFE_OpenCL *)fem->data;
+
+    PetscCall(PetscLogHandlerGetEventPerfInfo(h, PETSC_DEFAULT, ocl->residualEvent, &eventInfo));
+    eventInfo->count++;
+    eventInfo->time += time;
+    eventInfo->flops += flops;
+  }
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 static PetscErrorCode PetscFEIntegrateResidual_OpenCL(PetscDS prob, PetscFormKey key, PetscInt Ne, PetscFEGeom *cgeom, const PetscScalar coefficients[], const PetscScalar coefficients_t[], PetscDS probAux, const PetscScalar coefficientsAux[], PetscReal t, PetscScalar elemVec[])
@@ -544,7 +546,7 @@ static PetscErrorCode PetscFEIntegrateResidual_OpenCL(PetscDS prob, PetscFormKey
   ocl = (PetscFE_OpenCL *)fem->data;
   if (!Ne) {
     PetscCall(PetscFEOpenCLLogResidual(fem, 0.0, 0.0));
-    PetscFunctionReturn(0);
+    PetscFunctionReturn(PETSC_SUCCESS);
   }
   PetscCall(PetscFEGetSpatialDimension(fem, &dim));
   PetscCall(PetscFEGetQuadrature(fem, &q));
@@ -561,7 +563,7 @@ static PetscErrorCode PetscFEIntegrateResidual_OpenCL(PetscDS prob, PetscFormKey
   /* Calculate layout */
   if (Ne % (N_cb * N_bc)) { /* Remainder cells */
     PetscCall(PetscFEIntegrateResidual_Basic(prob, key, Ne, cgeom, coefficients, coefficients_t, probAux, coefficientsAux, t, elemVec));
-    PetscFunctionReturn(0);
+    PetscFunctionReturn(PETSC_SUCCESS);
   }
   PetscCall(PetscFEOpenCLCalculateGrid(fem, Ne, N_cb * N_bc, &x, &y, &z));
   local_work_size[0]  = N_bc * N_comp;
@@ -728,11 +730,11 @@ static PetscErrorCode PetscFEIntegrateResidual_OpenCL(PetscDS prob, PetscFormKey
   PetscCall(clReleaseMemObject(o_elemVec));
   PetscCall(clReleaseKernel(ocl_kernel));
   PetscCall(clReleaseProgram(ocl_prog));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 PETSC_INTERN PetscErrorCode PetscFESetUp_Basic(PetscFE);
-PETSC_INTERN PetscErrorCode PetscFECreateTabulation_Basic(PetscFE, PetscInt, const PetscReal[], PetscInt, PetscTabulation);
+PETSC_INTERN PetscErrorCode PetscFEComputeTabulation_Basic(PetscFE, PetscInt, const PetscReal[], PetscInt, PetscTabulation);
 
 static PetscErrorCode PetscFEInitialize_OpenCL(PetscFE fem)
 {
@@ -742,16 +744,16 @@ static PetscErrorCode PetscFEInitialize_OpenCL(PetscFE fem)
   fem->ops->view                    = NULL;
   fem->ops->destroy                 = PetscFEDestroy_OpenCL;
   fem->ops->getdimension            = PetscFEGetDimension_Basic;
-  fem->ops->createtabulation        = PetscFECreateTabulation_Basic;
+  fem->ops->computetabulation       = PetscFEComputeTabulation_Basic;
   fem->ops->integrateresidual       = PetscFEIntegrateResidual_OpenCL;
   fem->ops->integratebdresidual     = NULL /* PetscFEIntegrateBdResidual_OpenCL */;
   fem->ops->integratejacobianaction = NULL /* PetscFEIntegrateJacobianAction_OpenCL */;
   fem->ops->integratejacobian       = PetscFEIntegrateJacobian_Basic;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*MC
-  PETSCFEOPENCL = "opencl" - A PetscFE object that integrates using a vectorized OpenCL implementation
+  PETSCFEOPENCL = "opencl" - A `PetscFEType` that integrates using a vectorized OpenCL implementation
 
   Level: intermediate
 
@@ -781,7 +783,7 @@ PETSC_EXTERN PetscErrorCode PetscFECreate_OpenCL(PetscFE fem)
   PetscCheck(num_devices, PetscObjectComm((PetscObject)fem), PETSC_ERR_SUP, "No OpenCL device found.");
   ocl->dev_id = device_ids[0];
   /* Create context with one command queue */
-  ocl->ctx_id = clCreateContext(0, 1, &(ocl->dev_id), NULL, NULL, &err);
+  ocl->ctx_id = clCreateContext(0, 1, &ocl->dev_id, NULL, NULL, &err);
   PetscCall(err);
   ocl->queue_id = clCreateCommandQueue(ocl->ctx_id, ocl->dev_id, CL_QUEUE_PROFILING_ENABLE, &err);
   PetscCall(err);
@@ -793,19 +795,19 @@ PETSC_EXTERN PetscErrorCode PetscFECreate_OpenCL(PetscFE fem)
   ocl->op = LAPLACIAN;
 
   PetscCall(PetscFEInitialize_OpenCL(fem));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
-  PetscFEOpenCLSetRealType - Set the scalar type for running on the accelerator
+  PetscFEOpenCLSetRealType - Set the scalar type for running on the OpenCL accelerator
 
   Input Parameters:
-+ fem      - The PetscFE
++ fem      - The `PetscFE`
 - realType - The scalar type
 
   Level: developer
 
-.seealso: `PetscFEOpenCLGetRealType()`
+.seealso: `PetscFE`, `PetscFEOpenCLGetRealType()`
 @*/
 PetscErrorCode PetscFEOpenCLSetRealType(PetscFE fem, PetscDataType realType)
 {
@@ -814,21 +816,21 @@ PetscErrorCode PetscFEOpenCLSetRealType(PetscFE fem, PetscDataType realType)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(fem, PETSCFE_CLASSID, 1);
   ocl->realType = realType;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
-  PetscFEOpenCLGetRealType - Get the scalar type for running on the accelerator
+  PetscFEOpenCLGetRealType - Get the scalar type for running on the OpenCL accelerator
 
   Input Parameter:
-. fem      - The PetscFE
+. fem - The `PetscFE`
 
   Output Parameter:
 . realType - The scalar type
 
   Level: developer
 
-.seealso: `PetscFEOpenCLSetRealType()`
+.seealso: `PetscFE`, `PetscFEOpenCLSetRealType()`
 @*/
 PetscErrorCode PetscFEOpenCLGetRealType(PetscFE fem, PetscDataType *realType)
 {
@@ -836,9 +838,9 @@ PetscErrorCode PetscFEOpenCLGetRealType(PetscFE fem, PetscDataType *realType)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(fem, PETSCFE_CLASSID, 1);
-  PetscValidPointer(realType, 2);
+  PetscAssertPointer(realType, 2);
   *realType = ocl->realType;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 #endif /* PETSC_HAVE_OPENCL */
