@@ -2,15 +2,26 @@
 
 #ifdef PETSC_HAVE_EGADS
   #include <egads.h>
-/* Need to make EGADSLite header compatible */
-extern "C" int EGlite_getTopology(const ego, ego *, int *, int *, double *, int *, ego **, int **);
-extern "C" int EGlite_inTopology(const ego, const double *);
+  #include <egads_lite.h>
 #endif
 
 #if defined(PETSC_HAVE_TETGEN_TETLIBRARY_NEEDED)
   #define TETLIBRARY
 #endif
+#if defined(__clang__)
+  #pragma clang diagnostic push
+  #pragma clang diagnostic ignored "-Wunused-parameter"
+  #pragma clang diagnostic ignored "-Wzero-as-null-pointer-constant"
+#elif defined(__GNUC__) || defined(__GNUG__)
+  #pragma GCC diagnostic push
+  #pragma GCC diagnostic ignored "-Wunused-parameter"
+#endif
 #include <tetgen.h>
+#if defined(__clang__)
+  #pragma clang diagnostic pop
+#elif defined(__GNUC__) || defined(__GNUG__)
+  #pragma GCC diagnostic pop
+#endif
 
 /* This is to fix the tetrahedron orientation from TetGen */
 static PetscErrorCode DMPlexInvertCells_Tetgen(PetscInt numCells, PetscInt numCorners, PetscInt cells[])
@@ -26,7 +37,7 @@ static PetscErrorCode DMPlexInvertCells_Tetgen(PetscInt numCells, PetscInt numCo
   } while (0)
   for (coff = 0; coff < bound; coff += numCorners) SWAP(cells[coff], cells[coff + 1]);
 #undef SWAP
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 PETSC_EXTERN PetscErrorCode DMPlexGenerate_Tetgen(DM boundary, PetscBool interpolate, DM *dm)
@@ -101,12 +112,12 @@ PETSC_EXTERN PetscErrorCode DMPlexGenerate_Tetgen(DM boundary, PetscBool interpo
     in.facetmarkerlist = new int[in.numberoffacets];
     for (f = fStart; f < fEnd; ++f) {
       const PetscInt idx    = f - fStart;
-      PetscInt      *points = NULL, numPoints, p, numVertices = 0, v, val = -1;
+      PetscInt      *points = nullptr, numPoints, p, numVertices = 0, v, val = -1;
 
       in.facetlist[idx].numberofpolygons = 1;
       in.facetlist[idx].polygonlist      = new tetgenio::polygon[in.facetlist[idx].numberofpolygons];
       in.facetlist[idx].numberofholes    = 0;
-      in.facetlist[idx].holelist         = NULL;
+      in.facetlist[idx].holelist         = nullptr;
 
       PetscCall(DMPlexGetTransitiveClosure(boundary, f, PETSC_TRUE, &numPoints, &points));
       for (p = 0; p < numPoints * 2; p += 2) {
@@ -132,9 +143,9 @@ PETSC_EXTERN PetscErrorCode DMPlexGenerate_Tetgen(DM boundary, PetscBool interpo
 
     /* Take away 'Q' for verbose output */
 #ifdef PETSC_HAVE_EGADS
-    PetscCall(PetscStrcpy(args, "pqezQY"));
+    PetscCall(PetscStrncpy(args, "pYqezQY", sizeof(args)));
 #else
-    PetscCall(PetscStrcpy(args, "pqezQ"));
+    PetscCall(PetscStrncpy(args, "pqezQ", sizeof(args)));
 #endif
     if (mesh->tetgenOpts) {
       ::tetrahedralize(mesh->tetgenOpts, &in, &out);
@@ -146,8 +157,8 @@ PETSC_EXTERN PetscErrorCode DMPlexGenerate_Tetgen(DM boundary, PetscBool interpo
     const PetscInt numCorners  = 4;
     const PetscInt numCells    = out.numberoftetrahedra;
     const PetscInt numVertices = out.numberofpoints;
-    PetscReal     *meshCoords  = NULL;
-    PetscInt      *cells       = NULL;
+    PetscReal     *meshCoords  = nullptr;
+    PetscInt      *cells       = nullptr;
 
     if (sizeof(PetscReal) == sizeof(out.pointlist[0])) {
       meshCoords = (PetscReal *)out.pointlist;
@@ -204,6 +215,8 @@ PETSC_EXTERN PetscErrorCode DMPlexGenerate_Tetgen(DM boundary, PetscBool interpo
     }
 
     PetscCall(PetscObjectQuery((PetscObject)boundary, "EGADS Model", (PetscObject *)&modelObj));
+    if (!modelObj) { PetscCall(PetscObjectQuery((PetscObject)boundary, "EGADSlite Model", (PetscObject *)&modelObj)); }
+
     if (modelObj) {
 #ifdef PETSC_HAVE_EGADS
       DMLabel   bodyLabel;
@@ -213,20 +226,18 @@ PETSC_EXTERN PetscErrorCode DMPlexGenerate_Tetgen(DM boundary, PetscBool interpo
       ego       model, geom;
       int       Nb, oclass, mtype, *senses;
 
-      /* Get Attached EGADS Model from Original DMPlex */
+      PetscCall(DMPlexCopyEGADSInfo_Internal(boundary, *dm));
+
+      // Get Attached EGADS Model from Original DMPlex
       PetscCall(PetscObjectQuery((PetscObject)boundary, "EGADS Model", (PetscObject *)&modelObj));
       if (modelObj) {
         PetscCall(PetscContainerGetPointer(modelObj, (void **)&model));
-        PetscCall(EG_getTopology(model, &geom, &oclass, &mtype, NULL, &Nb, &bodies, &senses));
-        /* Transfer EGADS Model to Volumetric Mesh */
-        PetscCall(PetscObjectCompose((PetscObject)*dm, "EGADS Model", (PetscObject)modelObj));
+        PetscCall(EG_getTopology(model, &geom, &oclass, &mtype, nullptr, &Nb, &bodies, &senses));
       } else {
-        PetscCall(PetscObjectQuery((PetscObject)boundary, "EGADSLite Model", (PetscObject *)&modelObj));
+        PetscCall(PetscObjectQuery((PetscObject)boundary, "EGADSlite Model", (PetscObject *)&modelObj));
         if (modelObj) {
           PetscCall(PetscContainerGetPointer(modelObj, (void **)&model));
-          PetscCall(EGlite_getTopology(model, &geom, &oclass, &mtype, NULL, &Nb, &bodies, &senses));
-          /* Transfer EGADS Model to Volumetric Mesh */
-          PetscCall(PetscObjectCompose((PetscObject)*dm, "EGADSLite Model", (PetscObject)modelObj));
+          PetscCall(EGlite_getTopology(model, &geom, &oclass, &mtype, nullptr, &Nb, &bodies, &senses));
           islite = PETSC_TRUE;
         }
       }
@@ -242,11 +253,11 @@ PETSC_EXTERN PetscErrorCode DMPlexGenerate_Tetgen(DM boundary, PetscBool interpo
         PetscReal centroid[3] = {0., 0., 0.};
         PetscInt  b;
 
-        /* Deterimine what body the cell's centroid is located in */
+        /* Determine what body the cell's centroid is located in */
         if (!interpolate) {
           PetscSection coordSection;
           Vec          coordinates;
-          PetscScalar *coords = NULL;
+          PetscScalar *coords = nullptr;
           PetscInt     coordSize, s, d;
 
           PetscCall(DMGetCoordinatesLocal(*dm, &coordinates));
@@ -255,7 +266,7 @@ PETSC_EXTERN PetscErrorCode DMPlexGenerate_Tetgen(DM boundary, PetscBool interpo
           for (s = 0; s < coordSize; ++s)
             for (d = 0; d < dim; ++d) centroid[d] += coords[s * dim + d];
           PetscCall(DMPlexVecRestoreClosure(*dm, coordSection, coordinates, c, &coordSize, &coords));
-        } else PetscCall(DMPlexComputeCellGeometryFVM(*dm, c, NULL, centroid, NULL));
+        } else PetscCall(DMPlexComputeCellGeometryFVM(*dm, c, nullptr, centroid, nullptr));
         for (b = 0; b < Nb; ++b) {
           if (islite) {
             if (EGlite_inTopology(bodies[b], centroid) == EGADS_SUCCESS) break;
@@ -265,7 +276,7 @@ PETSC_EXTERN PetscErrorCode DMPlexGenerate_Tetgen(DM boundary, PetscBool interpo
         }
         if (b < Nb) {
           PetscInt  cval    = b, eVal, fVal;
-          PetscInt *closure = NULL, Ncl, cl;
+          PetscInt *closure = nullptr, Ncl, cl;
 
           PetscCall(DMLabelSetValue(bodyLabel, c, cval));
           PetscCall(DMPlexGetTransitiveClosure(*dm, c, PETSC_TRUE, &Ncl, &closure));
@@ -290,7 +301,7 @@ PETSC_EXTERN PetscErrorCode DMPlexGenerate_Tetgen(DM boundary, PetscBool interpo
     PetscCall(DMPlexSetRefinementUniform(*dm, PETSC_FALSE));
   }
   PetscCall(DMUniversalLabelDestroy(&universal));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 PETSC_EXTERN PetscErrorCode DMPlexRefine_Tetgen(DM dm, double *maxVolumes, DM *dmRefined)
@@ -365,12 +376,12 @@ PETSC_EXTERN PetscErrorCode DMPlexRefine_Tetgen(DM dm, double *maxVolumes, DM *d
     in.facetmarkerlist = new int[in.numberoffacets];
     for (f = fStart; f < fEnd; ++f) {
       const PetscInt idx    = f - fStart;
-      PetscInt      *points = NULL, numPoints, p, numVertices = 0, v, val;
+      PetscInt      *points = nullptr, numPoints, p, numVertices = 0, v, val;
 
       in.facetlist[idx].numberofpolygons = 1;
       in.facetlist[idx].polygonlist      = new tetgenio::polygon[in.facetlist[idx].numberofpolygons];
       in.facetlist[idx].numberofholes    = 0;
-      in.facetlist[idx].holelist         = NULL;
+      in.facetlist[idx].holelist         = nullptr;
 
       PetscCall(DMPlexGetTransitiveClosure(dm, f, PETSC_TRUE, &numPoints, &points));
       for (p = 0; p < numPoints * 2; p += 2) {
@@ -401,7 +412,7 @@ PETSC_EXTERN PetscErrorCode DMPlexRefine_Tetgen(DM dm, double *maxVolumes, DM *d
     in.tetrahedronlist = new int[in.numberoftetrahedra * in.numberofcorners];
     for (c = cStart; c < cEnd; ++c) {
       const PetscInt idx     = c - cStart;
-      PetscInt      *closure = NULL;
+      PetscInt      *closure = nullptr;
       PetscInt       closureSize;
 
       PetscCall(DMPlexGetTransitiveClosure(dm, c, PETSC_TRUE, &closureSize, &closure));
@@ -415,17 +426,17 @@ PETSC_EXTERN PetscErrorCode DMPlexRefine_Tetgen(DM dm, double *maxVolumes, DM *d
     char args[32];
 
     /* Take away 'Q' for verbose output */
-    PetscCall(PetscStrcpy(args, "qezQra"));
+    PetscCall(PetscStrncpy(args, "qezQra", sizeof(args)));
     ::tetrahedralize(args, &in, &out);
   }
 
-  in.tetrahedronvolumelist = NULL;
+  in.tetrahedronvolumelist = nullptr;
   {
     const PetscInt numCorners  = 4;
     const PetscInt numCells    = out.numberoftetrahedra;
     const PetscInt numVertices = out.numberofpoints;
-    PetscReal     *meshCoords  = NULL;
-    PetscInt      *cells       = NULL;
+    PetscReal     *meshCoords  = nullptr;
+    PetscInt      *cells       = nullptr;
     PetscBool      interpolate = isInterpolated == DMPLEX_INTERPOLATED_FULL ? PETSC_TRUE : PETSC_FALSE;
 
     if (sizeof(PetscReal) == sizeof(out.pointlist[0])) {
@@ -494,20 +505,18 @@ PETSC_EXTERN PetscErrorCode DMPlexRefine_Tetgen(DM dm, double *maxVolumes, DM *d
       ego       model, geom;
       int       Nb, oclass, mtype, *senses;
 
+      PetscCall(DMPlexCopyEGADSInfo_Internal(dm, *dmRefined));
+
       /* Get Attached EGADS Model from Original DMPlex */
       PetscCall(PetscObjectQuery((PetscObject)dm, "EGADS Model", (PetscObject *)&modelObj));
       if (modelObj) {
         PetscCall(PetscContainerGetPointer(modelObj, (void **)&model));
-        PetscCall(EG_getTopology(model, &geom, &oclass, &mtype, NULL, &Nb, &bodies, &senses));
-        /* Transfer EGADS Model to Volumetric Mesh */
-        PetscCall(PetscObjectCompose((PetscObject)*dmRefined, "EGADS Model", (PetscObject)modelObj));
+        PetscCall(EG_getTopology(model, &geom, &oclass, &mtype, nullptr, &Nb, &bodies, &senses));
       } else {
-        PetscCall(PetscObjectQuery((PetscObject)dm, "EGADSLite Model", (PetscObject *)&modelObj));
+        PetscCall(PetscObjectQuery((PetscObject)dm, "EGADSlite Model", (PetscObject *)&modelObj));
         if (modelObj) {
           PetscCall(PetscContainerGetPointer(modelObj, (void **)&model));
-          PetscCall(EGlite_getTopology(model, &geom, &oclass, &mtype, NULL, &Nb, &bodies, &senses));
-          /* Transfer EGADS Model to Volumetric Mesh */
-          PetscCall(PetscObjectCompose((PetscObject)*dmRefined, "EGADSLite Model", (PetscObject)modelObj));
+          PetscCall(EGlite_getTopology(model, &geom, &oclass, &mtype, nullptr, &Nb, &bodies, &senses));
           islite = PETSC_TRUE;
         }
       }
@@ -523,11 +532,11 @@ PETSC_EXTERN PetscErrorCode DMPlexRefine_Tetgen(DM dm, double *maxVolumes, DM *d
         PetscReal centroid[3] = {0., 0., 0.};
         PetscInt  b;
 
-        /* Deterimine what body the cell's centroid is located in */
+        /* Determine what body the cell's centroid is located in */
         if (!interpolate) {
           PetscSection coordSection;
           Vec          coordinates;
-          PetscScalar *coords = NULL;
+          PetscScalar *coords = nullptr;
           PetscInt     coordSize, s, d;
 
           PetscCall(DMGetCoordinatesLocal(*dmRefined, &coordinates));
@@ -536,7 +545,7 @@ PETSC_EXTERN PetscErrorCode DMPlexRefine_Tetgen(DM dm, double *maxVolumes, DM *d
           for (s = 0; s < coordSize; ++s)
             for (d = 0; d < dim; ++d) centroid[d] += coords[s * dim + d];
           PetscCall(DMPlexVecRestoreClosure(*dmRefined, coordSection, coordinates, c, &coordSize, &coords));
-        } else PetscCall(DMPlexComputeCellGeometryFVM(*dmRefined, c, NULL, centroid, NULL));
+        } else PetscCall(DMPlexComputeCellGeometryFVM(*dmRefined, c, nullptr, centroid, nullptr));
         for (b = 0; b < Nb; ++b) {
           if (islite) {
             if (EGlite_inTopology(bodies[b], centroid) == EGADS_SUCCESS) break;
@@ -546,7 +555,7 @@ PETSC_EXTERN PetscErrorCode DMPlexRefine_Tetgen(DM dm, double *maxVolumes, DM *d
         }
         if (b < Nb) {
           PetscInt  cval    = b, eVal, fVal;
-          PetscInt *closure = NULL, Ncl, cl;
+          PetscInt *closure = nullptr, Ncl, cl;
 
           PetscCall(DMLabelSetValue(bodyLabel, c, cval));
           PetscCall(DMPlexGetTransitiveClosure(*dmRefined, c, PETSC_TRUE, &Ncl, &closure));
@@ -570,5 +579,5 @@ PETSC_EXTERN PetscErrorCode DMPlexRefine_Tetgen(DM dm, double *maxVolumes, DM *d
     }
     PetscCall(DMPlexSetRefinementUniform(*dmRefined, PETSC_FALSE));
   }
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
