@@ -1,12 +1,12 @@
 /*
   This file defines an "generalized" additive Schwarz preconditioner for any Mat implementation.
-  In this version each MPI rank may intersect multiple subdomains and any subdomain may
-  intersect multiple MPI ranks.  Intersections of subdomains with MPI ranks are called *local
+  In this version, each MPI process may intersect multiple subdomains and any subdomain may
+  intersect multiple MPI processes.  Intersections of subdomains with MPI processes are called *local
   subdomains*.
 
        N    - total number of distinct global subdomains  (set explicitly in PCGASMSetTotalSubdomains() or implicitly PCGASMSetSubdomains() and then calculated in PCSetUp_GASM())
-       n    - actual number of local subdomains on this rank (set in PCGASMSetSubdomains() or calculated in PCGASMSetTotalSubdomains())
-       nmax - maximum number of local subdomains per rank    (calculated in PCSetUp_GASM())
+       n    - actual number of local subdomains on this process (set in `PCGASMSetSubdomains()` or calculated in `PCGASMSetTotalSubdomains()`)
+       nmax - maximum number of local subdomains per process (calculated in PCSetUp_GASM())
 */
 #include <petsc/private/pcimpl.h> /*I "petscpc.h" I*/
 #include <petscdm.h>
@@ -59,57 +59,59 @@ static PetscErrorCode PCGASMSubdomainView_Private(PC pc, PetscInt i, PetscViewer
   char           *cidx;
 
   PetscFunctionBegin;
-  PetscCheck(i >= 0 && i < osm->n, PetscObjectComm((PetscObject)viewer), PETSC_ERR_ARG_WRONG, "Invalid subdomain %" PetscInt_FMT ": must nonnegative and less than %" PetscInt_FMT, i, osm->n);
+  PetscCheck(i >= -1 && i < osm->n, PetscObjectComm((PetscObject)viewer), PETSC_ERR_ARG_WRONG, "Invalid subdomain %" PetscInt_FMT ": must nonnegative and less than %" PetscInt_FMT, i, osm->n);
+
   /* Inner subdomains. */
-  PetscCall(ISGetLocalSize(osm->iis[i], &nidx));
   /*
    No more than 15 characters per index plus a space.
    PetscViewerStringSPrintf requires a string of size at least 2, so use (nidx+1) instead of nidx,
    in case nidx == 0. That will take care of the space for the trailing '\0' as well.
    For nidx == 0, the whole string 16 '\0'.
    */
-#define len 16 * (nidx + 1) + 1
-  PetscCall(PetscMalloc1(len, &cidx));
-  PetscCall(PetscViewerStringOpen(PETSC_COMM_SELF, cidx, len, &sviewer));
-#undef len
-  PetscCall(ISGetIndices(osm->iis[i], &idx));
-  for (j = 0; j < nidx; ++j) PetscCall(PetscViewerStringSPrintf(sviewer, "%" PetscInt_FMT " ", idx[j]));
-  PetscCall(ISRestoreIndices(osm->iis[i], &idx));
-  PetscCall(PetscViewerDestroy(&sviewer));
   PetscCall(PetscViewerASCIIPrintf(viewer, "Inner subdomain:\n"));
   PetscCall(PetscViewerFlush(viewer));
   PetscCall(PetscViewerASCIIPushSynchronized(viewer));
-  PetscCall(PetscViewerASCIISynchronizedPrintf(viewer, "%s", cidx));
+  if (i > -1) {
+    PetscCall(ISGetLocalSize(osm->iis[i], &nidx));
+    PetscCall(PetscMalloc1(16 * (nidx + 1) + 1, &cidx));
+    PetscCall(PetscViewerStringOpen(PETSC_COMM_SELF, cidx, 16 * (nidx + 1) + 1, &sviewer));
+    PetscCall(ISGetIndices(osm->iis[i], &idx));
+    for (j = 0; j < nidx; ++j) PetscCall(PetscViewerStringSPrintf(sviewer, "%" PetscInt_FMT " ", idx[j]));
+    PetscCall(ISRestoreIndices(osm->iis[i], &idx));
+    PetscCall(PetscViewerDestroy(&sviewer));
+    PetscCall(PetscViewerASCIISynchronizedPrintf(viewer, "%s", cidx));
+    PetscCall(PetscFree(cidx));
+  }
   PetscCall(PetscViewerFlush(viewer));
   PetscCall(PetscViewerASCIIPopSynchronized(viewer));
   PetscCall(PetscViewerASCIIPrintf(viewer, "\n"));
   PetscCall(PetscViewerFlush(viewer));
-  PetscCall(PetscFree(cidx));
+
   /* Outer subdomains. */
-  PetscCall(ISGetLocalSize(osm->ois[i], &nidx));
   /*
    No more than 15 characters per index plus a space.
    PetscViewerStringSPrintf requires a string of size at least 2, so use (nidx+1) instead of nidx,
    in case nidx == 0. That will take care of the space for the trailing '\0' as well.
    For nidx == 0, the whole string 16 '\0'.
    */
-#define len 16 * (nidx + 1) + 1
-  PetscCall(PetscMalloc1(len, &cidx));
-  PetscCall(PetscViewerStringOpen(PETSC_COMM_SELF, cidx, len, &sviewer));
-#undef len
-  PetscCall(ISGetIndices(osm->ois[i], &idx));
-  for (j = 0; j < nidx; ++j) PetscCall(PetscViewerStringSPrintf(sviewer, "%" PetscInt_FMT " ", idx[j]));
-  PetscCall(PetscViewerDestroy(&sviewer));
-  PetscCall(ISRestoreIndices(osm->ois[i], &idx));
   PetscCall(PetscViewerASCIIPrintf(viewer, "Outer subdomain:\n"));
   PetscCall(PetscViewerFlush(viewer));
   PetscCall(PetscViewerASCIIPushSynchronized(viewer));
-  PetscCall(PetscViewerASCIISynchronizedPrintf(viewer, "%s", cidx));
+  if (i > -1) {
+    PetscCall(ISGetLocalSize(osm->ois[i], &nidx));
+    PetscCall(PetscMalloc1(16 * (nidx + 1) + 1, &cidx));
+    PetscCall(PetscViewerStringOpen(PETSC_COMM_SELF, cidx, 16 * (nidx + 1) + 1, &sviewer));
+    PetscCall(ISGetIndices(osm->ois[i], &idx));
+    for (j = 0; j < nidx; ++j) PetscCall(PetscViewerStringSPrintf(sviewer, "%" PetscInt_FMT " ", idx[j]));
+    PetscCall(PetscViewerDestroy(&sviewer));
+    PetscCall(ISRestoreIndices(osm->ois[i], &idx));
+    PetscCall(PetscViewerASCIISynchronizedPrintf(viewer, "%s", cidx));
+    PetscCall(PetscFree(cidx));
+  }
   PetscCall(PetscViewerFlush(viewer));
   PetscCall(PetscViewerASCIIPopSynchronized(viewer));
   PetscCall(PetscViewerASCIIPrintf(viewer, "\n"));
   PetscCall(PetscViewerFlush(viewer));
-  PetscCall(PetscFree(cidx));
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
@@ -120,7 +122,7 @@ static PetscErrorCode PCGASMPrintSubdomains(PC pc)
   char        fname[PETSC_MAX_PATH_LEN + 1];
   PetscInt    l, d, count;
   PetscBool   found;
-  PetscViewer viewer, sviewer = NULL;
+  PetscViewer viewer;
   PetscInt   *numbering, *permutation; /* global numbering of locally-supported subdomains and the permutation from the local ordering */
 
   PetscFunctionBegin;
@@ -141,14 +143,10 @@ static PetscErrorCode PCGASMPrintSubdomains(PC pc)
     /* Now let subdomains go one at a time in the global numbering order and print their subdomain/solver info. */
     if (l < osm->n) {
       d = permutation[l]; /* d is the local number of the l-th smallest (in the global ordering) among the locally supported subdomains */
-      if (numbering[d] == count) {
-        PetscCall(PetscViewerGetSubViewer(viewer, ((PetscObject)osm->ois[d])->comm, &sviewer));
-        PetscCall(PCGASMSubdomainView_Private(pc, d, sviewer));
-        PetscCall(PetscViewerRestoreSubViewer(viewer, ((PetscObject)osm->ois[d])->comm, &sviewer));
-        ++l;
-      }
-    }
-    PetscCallMPI(MPI_Barrier(PetscObjectComm((PetscObject)pc)));
+      if (numbering[d] == count) l++;
+      else d = -1;
+    } else d = -1;
+    PetscCall(PCGASMSubdomainView_Private(pc, d, viewer));
   }
   PetscCall(PetscFree2(numbering, permutation));
   PetscCall(PetscViewerDestroy(&viewer));
@@ -200,8 +198,6 @@ static PetscErrorCode PCView_GASM(PC pc, PetscViewer viewer)
     PetscCall(PetscViewerASCIIPrintf(viewer, "  Subdomain solver info is as follows:\n"));
     PetscCall(PetscViewerASCIIPushTab(viewer));
     PetscCall(PetscViewerASCIIPrintf(viewer, "  - - - - - - - - - - - - - - - - - -\n"));
-    /* Make sure that everybody waits for the banner to be printed. */
-    PetscCallMPI(MPI_Barrier(PetscObjectComm((PetscObject)viewer)));
     /* Now let subdomains go one at a time in the global numbering order and print their subdomain/solver info. */
     PetscCall(PCGASMComputeGlobalSubdomainNumbering_Private(pc, &numbering, &permutation));
     l = 0;
@@ -216,19 +212,26 @@ static PetscErrorCode PCView_GASM(PC pc, PetscViewer viewer)
           PetscCall(ISGetLocalSize(osm->ois[d], &bsz));
           PetscCall(PetscViewerASCIISynchronizedPrintf(sviewer, "  [%d|%d] (subcomm [%d|%d]) local subdomain number %" PetscInt_FMT ", local size = %" PetscInt_FMT "\n", rank, size, srank, ssize, d, bsz));
           PetscCall(PetscViewerFlush(sviewer));
+          PetscCall(PetscViewerASCIIPushTab(sviewer));
           if (view_subdomains) PetscCall(PCGASMSubdomainView_Private(pc, d, sviewer));
           if (!pc->setupcalled) {
-            PetscCall(PetscViewerASCIIPrintf(sviewer, "  Solver not set up yet: PCSetUp() not yet called\n"));
+            PetscCall(PetscViewerASCIISynchronizedPrintf(sviewer, "  Solver not set up yet: PCSetUp() not yet called\n"));
           } else {
             PetscCall(KSPView(osm->ksp[d], sviewer));
           }
+          PetscCall(PetscViewerASCIIPopTab(sviewer));
           PetscCall(PetscViewerASCIIPrintf(sviewer, "  - - - - - - - - - - - - - - - - - -\n"));
           PetscCall(PetscViewerFlush(sviewer));
           PetscCall(PetscViewerRestoreSubViewer(viewer, ((PetscObject)osm->ois[d])->comm, &sviewer));
           ++l;
+        } else {
+          PetscCall(PetscViewerGetSubViewer(viewer, PETSC_COMM_SELF, &sviewer));
+          PetscCall(PetscViewerRestoreSubViewer(viewer, PETSC_COMM_SELF, &sviewer));
         }
+      } else {
+        PetscCall(PetscViewerGetSubViewer(viewer, PETSC_COMM_SELF, &sviewer));
+        PetscCall(PetscViewerRestoreSubViewer(viewer, PETSC_COMM_SELF, &sviewer));
       }
-      PetscCallMPI(MPI_Barrier(PetscObjectComm((PetscObject)pc)));
     }
     PetscCall(PetscFree2(numbering, permutation));
     PetscCall(PetscViewerASCIIPopTab(viewer));
@@ -277,10 +280,10 @@ static PetscErrorCode PCGASMSetHierarchicalPartitioning(PC pc)
   PetscCall(ISGetLocalSize(fromrows, &fromrows_localsize));
   PetscCall(MatPartitioningDestroy(&part));
   PetscCall(MatCreateVecs(pc->pmat, &outervec, NULL));
-  PetscCall(VecCreateMPI(comm, fromrows_localsize, PETSC_DETERMINE, &(osm->pcx)));
-  PetscCall(VecDuplicate(osm->pcx, &(osm->pcy)));
-  PetscCall(VecScatterCreate(osm->pcx, NULL, outervec, fromrows, &(osm->pctoouter)));
-  PetscCall(MatCreateSubMatrix(pc->pmat, fromrows, fromrows, MAT_INITIAL_MATRIX, &(osm->permutationP)));
+  PetscCall(VecCreateMPI(comm, fromrows_localsize, PETSC_DETERMINE, &osm->pcx));
+  PetscCall(VecDuplicate(osm->pcx, &osm->pcy));
+  PetscCall(VecScatterCreate(osm->pcx, NULL, outervec, fromrows, &osm->pctoouter));
+  PetscCall(MatCreateSubMatrix(pc->pmat, fromrows, fromrows, MAT_INITIAL_MATRIX, &osm->permutationP));
   PetscCall(PetscObjectReference((PetscObject)fromrows));
   osm->permutationIS = fromrows;
   osm->pcmat         = pc->pmat;
@@ -365,7 +368,7 @@ static PetscErrorCode PCSetUp_GASM(PC pc)
           PetscCall(ISDuplicate(osm->iis[i], (osm->ois) + i));
           PetscCall(ISCopy(osm->iis[i], osm->ois[i]));
         } else {
-          PetscCall(PetscObjectReference((PetscObject)((osm->iis)[i])));
+          PetscCall(PetscObjectReference((PetscObject)osm->iis[i]));
           osm->ois[i] = osm->iis[i];
         }
       }
@@ -377,10 +380,10 @@ static PetscErrorCode PCSetUp_GASM(PC pc)
 
     /* Now the subdomains are defined.  Determine their global and max local numbers, if necessary. */
     if (osm->nmax == PETSC_DETERMINE) {
-      PetscMPIInt inwork, outwork;
+      PetscInt inwork, outwork;
       /* determine global number of subdomains and the max number of local subdomains */
       inwork = osm->n;
-      PetscCall(MPIU_Allreduce(&inwork, &outwork, 1, MPI_INT, MPI_MAX, PetscObjectComm((PetscObject)pc)));
+      PetscCallMPI(MPIU_Allreduce(&inwork, &outwork, 1, MPIU_INT, MPI_MAX, PetscObjectComm((PetscObject)pc)));
       osm->nmax = outwork;
     }
     if (osm->N == PETSC_DETERMINE) {
@@ -416,13 +419,13 @@ static PetscErrorCode PCSetUp_GASM(PC pc)
       PetscCall(ISRestoreIndices(osm->ois[i], &oidxi));
       on += oni;
     }
-    PetscCall(ISCreateGeneral(((PetscObject)(pc))->comm, on, oidx, PETSC_OWN_POINTER, &gois));
+    PetscCall(ISCreateGeneral(((PetscObject)pc)->comm, on, oidx, PETSC_OWN_POINTER, &gois));
     nTotalInnerIndices = 0;
     for (i = 0; i < osm->n; i++) {
       PetscCall(ISGetLocalSize(osm->iis[i], &nInnerIndices));
       nTotalInnerIndices += nInnerIndices;
     }
-    PetscCall(VecCreateMPI(((PetscObject)(pc))->comm, nTotalInnerIndices, PETSC_DETERMINE, &x));
+    PetscCall(VecCreateMPI(((PetscObject)pc)->comm, nTotalInnerIndices, PETSC_DETERMINE, &x));
     PetscCall(VecDuplicate(x, &y));
 
     PetscCall(VecCreateMPI(PetscObjectComm((PetscObject)pc), on, PETSC_DECIDE, &osm->gx));
@@ -430,7 +433,7 @@ static PetscErrorCode PCSetUp_GASM(PC pc)
     PetscCall(VecGetOwnershipRange(osm->gx, &gostart, NULL));
     PetscCall(ISCreateStride(PetscObjectComm((PetscObject)pc), on, gostart, 1, &goid));
     /* gois might indices not on local */
-    PetscCall(VecScatterCreate(x, gois, osm->gx, goid, &(osm->gorestriction)));
+    PetscCall(VecScatterCreate(x, gois, osm->gx, goid, &osm->gorestriction));
     PetscCall(PetscMalloc1(osm->n, &numbering));
     PetscCall(PetscObjectsListGetGlobalNumbering(PetscObjectComm((PetscObject)pc), osm->n, (PetscObject *)osm->ois, NULL, numbering));
     PetscCall(VecDestroy(&x));
@@ -502,8 +505,8 @@ static PetscErrorCode PCSetUp_GASM(PC pc)
       PetscCall(ISGetLocalSize(osm->ois[i], &oni));
       /* on a sub communicator */
       PetscCall(ISGetSize(osm->ois[i], &oNi));
-      PetscCall(VecCreateMPIWithArray(((PetscObject)(osm->ois[i]))->comm, 1, oni, oNi, gxarray + on, &osm->x[i]));
-      PetscCall(VecCreateMPIWithArray(((PetscObject)(osm->ois[i]))->comm, 1, oni, oNi, gyarray + on, &osm->y[i]));
+      PetscCall(VecCreateMPIWithArray(((PetscObject)osm->ois[i])->comm, 1, oni, oNi, gxarray + on, &osm->x[i]));
+      PetscCall(VecCreateMPIWithArray(((PetscObject)osm->ois[i])->comm, 1, oni, oNi, gyarray + on, &osm->y[i]));
     }
     PetscCall(VecRestoreArray(osm->gx, &gxarray));
     PetscCall(VecRestoreArray(osm->gy, &gyarray));
@@ -511,7 +514,7 @@ static PetscErrorCode PCSetUp_GASM(PC pc)
     PetscCall(PetscMalloc1(osm->n, &osm->ksp));
     for (i = 0; i < osm->n; i++) {
       char subprefix[PETSC_MAX_PATH_LEN + 1];
-      PetscCall(KSPCreate(((PetscObject)(osm->ois[i]))->comm, &ksp));
+      PetscCall(KSPCreate(((PetscObject)osm->ois[i])->comm, &ksp));
       PetscCall(KSPSetNestLevel(ksp, pc->kspnestlevel));
       PetscCall(KSPSetErrorIfNotConverged(ksp, pc->erroriffailure));
       PetscCall(PetscObjectIncrementTabLevel((PetscObject)ksp, (PetscObject)pc, 1));
@@ -827,11 +830,11 @@ static PetscErrorCode PCReset_GASM(PC pc)
     osm->N    = PETSC_DETERMINE;
     osm->nmax = PETSC_DETERMINE;
   }
-  if (osm->pctoouter) PetscCall(VecScatterDestroy(&(osm->pctoouter)));
-  if (osm->permutationIS) PetscCall(ISDestroy(&(osm->permutationIS)));
-  if (osm->pcx) PetscCall(VecDestroy(&(osm->pcx)));
-  if (osm->pcy) PetscCall(VecDestroy(&(osm->pcy)));
-  if (osm->permutationP) PetscCall(MatDestroy(&(osm->permutationP)));
+  if (osm->pctoouter) PetscCall(VecScatterDestroy(&osm->pctoouter));
+  if (osm->permutationIS) PetscCall(ISDestroy(&osm->permutationIS));
+  if (osm->pcx) PetscCall(VecDestroy(&osm->pcx));
+  if (osm->pcy) PetscCall(VecDestroy(&osm->pcy));
+  if (osm->permutationP) PetscCall(MatDestroy(&osm->permutationP));
   if (osm->pcmat) PetscCall(MatDestroy(&osm->pcmat));
   PetscFunctionReturn(PETSC_SUCCESS);
 }
@@ -860,7 +863,7 @@ static PetscErrorCode PCDestroy_GASM(PC pc)
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-static PetscErrorCode PCSetFromOptions_GASM(PC pc, PetscOptionItems *PetscOptionsObject)
+static PetscErrorCode PCSetFromOptions_GASM(PC pc, PetscOptionItems PetscOptionsObject)
 {
   PC_GASM   *osm = (PC_GASM *)pc->data;
   PetscInt   blocks, ovl;
@@ -896,7 +899,7 @@ static PetscErrorCode PCSetFromOptions_GASM(PC pc, PetscOptionItems *PetscOption
 
   Level: beginner
 
-.seealso: `PCGASM`, `PCGASMSetSubdomains()`, `PCGASMSetOverlap()`
+.seealso: [](ch_ksp), `PCGASM`, `PCGASMSetSubdomains()`, `PCGASMSetOverlap()`
           `PCGASMCreateSubdomains2D()`
 @*/
 PetscErrorCode PCGASMSetTotalSubdomains(PC pc, PetscInt N)
@@ -965,7 +968,7 @@ static PetscErrorCode PCGASMSetSubdomains_GASM(PC pc, PetscInt n, IS iis[], IS o
     /* check if the inner indices cover and only cover the local portion of the preconditioning matrix */
     PetscCall(MatGetOwnershipRange(pc->pmat, &rstart, &rend));
     PetscCall(PetscCalloc1(rend - rstart, &covered));
-    /* check if the current MPI rank owns indices from others */
+    /* check if the current MPI process owns indices from others */
     for (i = 0; i < n; i++) {
       PetscCall(ISGetIndices(osm->iis[i], &indices));
       PetscCall(ISGetLocalSize(osm->iis[i], &lsize));
@@ -1015,7 +1018,7 @@ static PetscErrorCode PCGASMSetSortIndices_GASM(PC pc, PetscBool doSort)
 }
 
 /*
-   FIXME: This routine might need to be modified now that multiple ranks per subdomain are allowed.
+   FIXME: This routine might need to be modified now that multiple processes per subdomain are allowed.
         In particular, it would upset the global subdomain number calculation.
 */
 static PetscErrorCode PCGASMGetSubKSP_GASM(PC pc, PetscInt *n, PetscInt *first, KSP **ksp)
@@ -1039,18 +1042,19 @@ static PetscErrorCode PCGASMGetSubKSP_GASM(PC pc, PetscInt *n, PetscInt *first,
   PetscFunctionReturn(PETSC_SUCCESS);
 } /* PCGASMGetSubKSP_GASM() */
 
-/*@C
-  PCGASMSetSubdomains - Sets the subdomains for this MPI rank
-  for the additive Schwarz preconditioner with multiple MPI ranks per subdomain, `PCGASM`
+/*@
+  PCGASMSetSubdomains - Sets the subdomains for this MPI process
+  for the additive Schwarz preconditioner with multiple MPI processes per subdomain, `PCGASM`
 
   Collective
 
   Input Parameters:
 + pc  - the preconditioner object
-. n   - the number of subdomains for this MPI rank
-. iis - the index sets that define the inner subdomains (or `NULL` for PETSc to determine subdomains)
+. n   - the number of subdomains for this MPI process
+. iis - the index sets that define the inner subdomains (or `NULL` for PETSc to determine subdomains), the `iis` array is
+        copied so may be freed after this call.
 - ois - the index sets that define the outer subdomains (or `NULL` to use the same as `iis`, or to construct by expanding `iis` by
-          the requested overlap)
+        the requested overlap), the `ois` array is copied so may be freed after this call.
 
   Level: advanced
 
@@ -1062,19 +1066,19 @@ static PetscErrorCode PCGASMGetSubKSP_GASM(PC pc, PetscInt *n, PetscInt *first,
   Outer subdomains are those where the residual necessary to obtain the
   corrections is obtained (see `PCGASMType` for the use of inner/outer subdomains).
 
-  Both inner and outer subdomains can extend over several MPI ranks.
-  This rank's portion of a subdomain is known as a local subdomain.
+  Both inner and outer subdomains can extend over several MPI processes.
+  This process' portion of a subdomain is known as a local subdomain.
 
-  Inner subdomains can not overlap with each other, do not have any entities from remote ranks,
-  and  have to cover the entire local subdomain owned by the current rank. The index sets on each
-  rank should be ordered such that the ith local subdomain is connected to the ith remote subdomain
-  on another MPI rank.
+  Inner subdomains can not overlap with each other, do not have any entities from remote processes,
+  and  have to cover the entire local subdomain owned by the current process. The index sets on each
+  process should be ordered such that the ith local subdomain is connected to the ith remote subdomain
+  on another MPI process.
 
-  By default the `PGASM` preconditioner uses 1 (local) subdomain per MPI rank.
+  By default the `PGASM` preconditioner uses 1 (local) subdomain per MPI process.
 
   The `iis` and `ois` arrays may be freed after this call using `PCGASMDestroySubdomains()`
 
-.seealso: `PCGASM`, `PCGASMSetOverlap()`, `PCGASMGetSubKSP()`, `PCGASMDestroySubdomains()`,
+.seealso: [](ch_ksp), `PCGASM`, `PCGASMSetOverlap()`, `PCGASMGetSubKSP()`, `PCGASMDestroySubdomains()`,
           `PCGASMCreateSubdomains2D()`, `PCGASMGetSubdomains()`
 @*/
 PetscErrorCode PCGASMSetSubdomains(PC pc, PetscInt n, IS iis[], IS ois[])
@@ -1090,7 +1094,7 @@ PetscErrorCode PCGASMSetSubdomains(PC pc, PetscInt n, IS iis[], IS ois[])
 
 /*@
   PCGASMSetOverlap - Sets the overlap between a pair of subdomains for the
-  additive Schwarz preconditioner `PCGASM`.  Either all or no MPI ranks in the
+  additive Schwarz preconditioner `PCGASM`.  Either all or no MPI processes in the
   pc communicator must call this routine.
 
   Logically Collective
@@ -1105,9 +1109,9 @@ PetscErrorCode PCGASMSetSubdomains(PC pc, PetscInt n, IS iis[], IS ois[])
   Level: intermediate
 
   Notes:
-  By default the `PCGASM` preconditioner uses 1 subdomain per rank.  To use
+  By default the `PCGASM` preconditioner uses 1 subdomain per process.  To use
   multiple subdomain per perocessor or "straddling" subdomains that intersect
-  multiple ranks use `PCGASMSetSubdomains()` (or option `-pc_gasm_total_subdomains` <n>).
+  multiple processes use `PCGASMSetSubdomains()` (or option `-pc_gasm_total_subdomains` <n>).
 
   The overlap defaults to 0, so if one desires that no additional
   overlap be computed beyond what may have been set with a call to
@@ -1120,7 +1124,7 @@ PetscErrorCode PCGASMSetSubdomains(PC pc, PetscInt n, IS iis[], IS ois[])
   `PCGASMSetSubdomains()`; the routine `PCGASMSetOverlap()` merely allows
   PETSc to extend that overlap further, if desired.
 
-.seealso: `PCGASM`, `PCGASMSetSubdomains()`, `PCGASMGetSubKSP()`,
+.seealso: [](ch_ksp), `PCGASM`, `PCGASMSetSubdomains()`, `PCGASMGetSubKSP()`,
           `PCGASMCreateSubdomains2D()`, `PCGASMGetSubdomains()`
 @*/
 PetscErrorCode PCGASMSetOverlap(PC pc, PetscInt ovl)
@@ -1146,9 +1150,9 @@ PetscErrorCode PCGASMSetOverlap(PC pc, PetscInt ovl)
 - type - variant of `PCGASM`, one of
 .vb
       `PC_GASM_BASIC`       - full interpolation and restriction
-      `PC_GASM_RESTRICT`    - full restriction, local MPI rank interpolation
-      `PC_GASM_INTERPOLATE` - full interpolation, local MPI rank restriction
-      `PC_GASM_NONE`        - local MPI rank restriction and interpolation
+      `PC_GASM_RESTRICT`    - full restriction, local MPI process interpolation
+      `PC_GASM_INTERPOLATE` - full interpolation, local MPI process restriction
+      `PC_GASM_NONE`        - local MPI process restriction and interpolation
 .ve
 
   Options Database Key:
@@ -1156,7 +1160,7 @@ PetscErrorCode PCGASMSetOverlap(PC pc, PetscInt ovl)
 
   Level: intermediate
 
-.seealso: `PCGASM`, `PCGASMSetSubdomains()`, `PCGASMGetSubKSP()`,
+.seealso: [](ch_ksp), `PCGASM`, `PCGASMSetSubdomains()`, `PCGASMGetSubKSP()`,
           `PCGASMCreateSubdomains2D()`, `PCASM`, `PCASMSetType()`
 @*/
 PetscErrorCode PCGASMSetType(PC pc, PCGASMType type)
@@ -1179,7 +1183,7 @@ PetscErrorCode PCGASMSetType(PC pc, PCGASMType type)
 
   Level: intermediate
 
-.seealso: `PCGASM`, `PCGASMSetSubdomains()`, `PCGASMGetSubKSP()`,
+.seealso: [](ch_ksp), `PCGASM`, `PCGASMSetSubdomains()`, `PCGASMGetSubKSP()`,
           `PCGASMCreateSubdomains2D()`
 @*/
 PetscErrorCode PCGASMSetSortIndices(PC pc, PetscBool doSort)
@@ -1192,8 +1196,7 @@ PetscErrorCode PCGASMSetSortIndices(PC pc, PetscBool doSort)
 }
 
 /*@C
-  PCGASMGetSubKSP - Gets the local `KSP` contexts for all subdomains on
-  this MPI rank.
+  PCGASMGetSubKSP - Gets the local `KSP` contexts for all subdomains on this MPI process.
 
   Collective iff first_local is requested
 
@@ -1201,9 +1204,8 @@ PetscErrorCode PCGASMSetSortIndices(PC pc, PetscBool doSort)
 . pc - the preconditioner context
 
   Output Parameters:
-+ n_local     - the number of blocks on this MPI rank or `NULL`
-. first_local - the global number of the first block on this rank or `NULL`,
-                 all ranks must request or all must pass `NULL`
++ n_local     - the number of blocks on this MPI process or `NULL`
+. first_local - the global number of the first block on this process or `NULL`, all processes must request or all must pass `NULL`
 - ksp         - the array of `KSP` contexts
 
   Level: advanced
@@ -1216,7 +1218,7 @@ PetscErrorCode PCGASMSetSortIndices(PC pc, PetscBool doSort)
 
   You must call `KSPSetUp()` before calling `PCGASMGetSubKSP()`.
 
-.seealso: `PCGASM`, `PCGASMSetSubdomains()`, `PCGASMSetOverlap()`,
+.seealso: [](ch_ksp), `PCGASM`, `PCGASMSetSubdomains()`, `PCGASMSetOverlap()`,
           `PCGASMCreateSubdomains2D()`,
 @*/
 PetscErrorCode PCGASMGetSubKSP(PC pc, PetscInt *n_local, PetscInt *first_local, KSP *ksp[])
@@ -1229,13 +1231,13 @@ PetscErrorCode PCGASMGetSubKSP(PC pc, PetscInt *n_local, PetscInt *first_local,
 
 /*MC
    PCGASM - Use the (restricted) additive Schwarz method, each block is (approximately) solved with
-           its own `KSP` object on a subset of MPI ranks
+           its own `KSP` object on a subset of MPI processes
 
    Options Database Keys:
-+  -pc_gasm_total_subdomains <n>  - Sets total number of local subdomains to be distributed among MPI ranks
-.  -pc_gasm_view_subdomains       - activates the printing of subdomain indices in `PCView()`, -ksp_view or -snes_view
-.  -pc_gasm_print_subdomains      - activates the printing of subdomain indices in `PCSetUp()`
-.  -pc_gasm_overlap <ovl>         - Sets overlap by which to (automatically) extend local subdomains
++  -pc_gasm_total_subdomains <n>                   - Sets total number of local subdomains to be distributed among the MPI processes
+.  -pc_gasm_view_subdomains                        - activates the printing of subdomain indices in `PCView()`, -ksp_view or -snes_view
+.  -pc_gasm_print_subdomains                       - activates the printing of subdomain indices in `PCSetUp()`
+.  -pc_gasm_overlap <ovl>                          - Sets overlap by which to (automatically) extend local subdomains
 -  -pc_gasm_type [basic,restrict,interpolate,none] - Sets `PCGASMType`
 
    Level: beginner
@@ -1248,13 +1250,9 @@ PetscErrorCode PCGASMGetSubKSP(PC pc, PetscInt *n_local, PetscInt *first_local,
    and set the options directly on the resulting `KSP` object (you can access its `PC`
    with `KSPGetPC()`)
 
-    References:
-+   * - M Dryja, OB Widlund, An additive variant of the Schwarz alternating method for the case of many subregions
-     Courant Institute, New York University Technical report
--   * - Barry Smith, Petter Bjorstad, and William Gropp, Domain Decompositions: Parallel Multilevel Methods for Elliptic Partial Differential Equations,
-    Cambridge University Press.
+   See {cite}`dryja1987additive` and {cite}`1sbg` for details on additive Schwarz algorithms
 
-.seealso: `PCCreate()`, `PCSetType()`, `PCType`, `PC`, `PCASM`, `PCGASMType`, `PCGASMSetType()`,
+.seealso: [](ch_ksp), `PCCreate()`, `PCSetType()`, `PCType`, `PC`, `PCASM`, `PCGASMType`, `PCGASMSetType()`,
           `PCBJACOBI`, `PCGASMGetSubKSP()`, `PCGASMSetSubdomains()`,
           `PCSetModifySubMatrices()`, `PCGASMSetOverlap()`, `PCGASMSetType()`
 M*/
@@ -1471,21 +1469,21 @@ PETSC_INTERN PetscErrorCode PCGASMCreateStraddlingSubdomains(Mat A, PetscInt N,
 - N - the number of global subdomains requested
 
   Output Parameters:
-+ n   - the number of subdomains created on this MPI rank
++ n   - the number of subdomains created on this MPI process
 - iis - the array of index sets defining the local inner subdomains (on which the correction is applied)
 
   Level: advanced
 
   Notes:
   When `N` >= A's communicator size, each subdomain is local -- contained within a single MPI process.
-  When `N` < size, the subdomains are 'straddling' (rank boundaries) and are no longer local.
+  When `N` < size, the subdomains are 'straddling' (process boundaries) and are no longer local.
   The resulting subdomains can be use in `PCGASMSetSubdomains`(pc,n,iss,`NULL`).  The overlapping
   outer subdomains will be automatically generated from these according to the requested amount of
   overlap; this is currently supported only with local subdomains.
 
   Use `PCGASMDestroySubdomains()` to free the array and the list of index sets.
 
-.seealso: `PCGASM`, `PCGASMSetSubdomains()`, `PCGASMDestroySubdomains()`
+.seealso: [](ch_ksp), `PCGASM`, `PCGASMSetSubdomains()`, `PCGASMDestroySubdomains()`
 @*/
 PetscErrorCode PCGASMCreateSubdomains(Mat A, PetscInt N, PetscInt *n, IS *iis[])
 {
@@ -1525,9 +1523,9 @@ PetscErrorCode PCGASMCreateSubdomains(Mat A, PetscInt N, PetscInt *n, IS *iis[])
   destroys each `IS` on the list, and then frees the list. At the end the
   list pointers are set to `NULL`.
 
-.seealso: `PCGASM`, `PCGASMCreateSubdomains()`, `PCGASMSetSubdomains()`
+.seealso: [](ch_ksp), `PCGASM`, `PCGASMCreateSubdomains()`, `PCGASMSetSubdomains()`
 @*/
-PetscErrorCode PCGASMDestroySubdomains(PetscInt n, IS **iis, IS **ois)
+PetscErrorCode PCGASMDestroySubdomains(PetscInt n, IS *iis[], IS *ois[])
 {
   PetscInt i;
 
@@ -1538,7 +1536,7 @@ PetscErrorCode PCGASMDestroySubdomains(PetscInt n, IS **iis, IS **ois)
     if (*ois) {
       PetscAssertPointer(*ois, 3);
       for (i = 0; i < n; i++) PetscCall(ISDestroy(&(*ois)[i]));
-      PetscCall(PetscFree((*ois)));
+      PetscCall(PetscFree(*ois));
     }
   }
   if (iis) {
@@ -1546,7 +1544,7 @@ PetscErrorCode PCGASMDestroySubdomains(PetscInt n, IS **iis, IS **ois)
     if (*iis) {
       PetscAssertPointer(*iis, 2);
       for (i = 0; i < n; i++) PetscCall(ISDestroy(&(*iis)[i]));
-      PetscCall(PetscFree((*iis)));
+      PetscCall(PetscFree(*iis));
     }
   }
   PetscFunctionReturn(PETSC_SUCCESS);
@@ -1605,13 +1603,10 @@ PetscErrorCode PCGASMDestroySubdomains(PetscInt n, IS **iis, IS **ois)
   Note:
   Use `PCGASMDestroySubdomains()` to free the index sets and the arrays
 
-  Fortran Notes:
-  The `IS` must be declared as an array of length long enough to hold `Nsub` entries
-
-.seealso: `PCGASM`, `PCGASMSetSubdomains()`, `PCGASMGetSubKSP()`, `PCGASMSetOverlap()`, `PCASMCreateSubdomains2D()`,
+.seealso: [](ch_ksp), `PCGASM`, `PCGASMSetSubdomains()`, `PCGASMGetSubKSP()`, `PCGASMSetOverlap()`, `PCASMCreateSubdomains2D()`,
           `PCGASMDestroySubdomains()`
 @*/
-PetscErrorCode PCGASMCreateSubdomains2D(PC pc, PetscInt M, PetscInt N, PetscInt Mdomains, PetscInt Ndomains, PetscInt dof, PetscInt overlap, PetscInt *nsub, IS **iis, IS **ois)
+PetscErrorCode PCGASMCreateSubdomains2D(PC pc, PetscInt M, PetscInt N, PetscInt Mdomains, PetscInt Ndomains, PetscInt dof, PetscInt overlap, PetscInt *nsub, IS *iis[], IS *ois[])
 {
   PetscMPIInt size, rank;
   PetscInt    i, j;
@@ -1754,7 +1749,7 @@ PetscErrorCode PCGASMCreateSubdomains2D(PC pc, PetscInt M, PetscInt N, PetscInt
           PetscCall(ISCreateGeneral(subcomm, nidx, idx, PETSC_OWN_POINTER, (*xis) + s));
           if (split) PetscCallMPI(MPI_Comm_free(&subcomm));
         } /* if (n[0]) */
-      }   /* for (q = 0; q < 2; ++q) */
+      } /* for (q = 0; q < 2; ++q) */
       if (n[0]) ++s;
       xstart += maxwidth;
     } /* for (i = 0; i < Mdomains; ++i) */
@@ -1764,7 +1759,7 @@ PetscErrorCode PCGASMCreateSubdomains2D(PC pc, PetscInt M, PetscInt N, PetscInt
 }
 
 /*@C
-  PCGASMGetSubdomains - Gets the subdomains supported on this MPI rank
+  PCGASMGetSubdomains - Gets the subdomains supported on this MPI process
   for the `PCGASM` additive Schwarz preconditioner.
 
   Not Collective
@@ -1773,9 +1768,9 @@ PetscErrorCode PCGASMCreateSubdomains2D(PC pc, PetscInt M, PetscInt N, PetscInt
 . pc - the preconditioner context
 
   Output Parameters:
-+ n   - the number of subdomains for this MPI rank (default value = 1)
-. iis - the index sets that define the inner subdomains (without overlap) supported on this rank (can be `NULL`)
-- ois - the index sets that define the outer subdomains (with overlap) supported on this rank (can be `NULL`)
++ n   - the number of subdomains for this MPI process (default value = 1)
+. iis - the index sets that define the inner subdomains (without overlap) supported on this process (can be `NULL`)
+- ois - the index sets that define the outer subdomains (with overlap) supported on this process (can be `NULL`)
 
   Level: advanced
 
@@ -1785,7 +1780,7 @@ PetscErrorCode PCGASMCreateSubdomains2D(PC pc, PetscInt M, PetscInt N, PetscInt
 
   The `IS` numbering is in the parallel, global numbering of the vector.
 
-.seealso: `PCGASM`, `PCGASMSetOverlap()`, `PCGASMGetSubKSP()`, `PCGASMCreateSubdomains2D()`,
+.seealso: [](ch_ksp), `PCGASM`, `PCGASMSetOverlap()`, `PCGASMGetSubKSP()`, `PCGASMCreateSubdomains2D()`,
           `PCGASMSetSubdomains()`, `PCGASMGetSubmatrices()`, `PCGASMDestroySubdomains()`
 @*/
 PetscErrorCode PCGASMGetSubdomains(PC pc, PetscInt *n, IS *iis[], IS *ois[])
@@ -1812,7 +1807,7 @@ PetscErrorCode PCGASMGetSubdomains(PC pc, PetscInt *n, IS *iis[], IS *ois[])
 }
 
 /*@C
-  PCGASMGetSubmatrices - Gets the local submatrices (for this MPI rank
+  PCGASMGetSubmatrices - Gets the local submatrices (for this MPI process
   only) for the `PCGASM` additive Schwarz preconditioner.
 
   Not Collective
@@ -1821,7 +1816,7 @@ PetscErrorCode PCGASMGetSubdomains(PC pc, PetscInt *n, IS *iis[], IS *ois[])
 . pc - the preconditioner context
 
   Output Parameters:
-+ n   - the number of matrices for this MPI rank (default value = 1)
++ n   - the number of matrices for this MPI process (default value = 1)
 - mat - the matrices
 
   Level: advanced
@@ -1830,7 +1825,7 @@ PetscErrorCode PCGASMGetSubdomains(PC pc, PetscInt *n, IS *iis[], IS *ois[])
   Matrices returned by this routine have the same communicators as the index sets (`IS`)
   used to define subdomains in `PCGASMSetSubdomains()`
 
-.seealso: `PCGASM`, `PCGASMSetOverlap()`, `PCGASMGetSubKSP()`,
+.seealso: [](ch_ksp), `PCGASM`, `PCGASMSetOverlap()`, `PCGASMGetSubKSP()`,
           `PCGASMCreateSubdomains2D()`, `PCGASMSetSubdomains()`, `PCGASMGetSubdomains()`
 @*/
 PetscErrorCode PCGASMGetSubmatrices(PC pc, PetscInt *n, Mat *mat[])
@@ -1872,7 +1867,7 @@ PetscErrorCode PCGASMGetSubmatrices(PC pc, PetscInt *n, Mat *mat[])
   so setting `PCGASMSetSubdomains()` with nontrivial subdomain ISs or any of `PCGASMSetTotalSubdomains()` and `PCGASMSetOverlap()`
   automatically turns the latter off.
 
-.seealso: `PCGASM`, `PCGASMGetUseDMSubdomains()`, `PCGASMSetSubdomains()`, `PCGASMSetOverlap()`
+.seealso: [](ch_ksp), `PCGASM`, `PCGASMGetUseDMSubdomains()`, `PCGASMSetSubdomains()`, `PCGASMSetOverlap()`
           `PCGASMCreateSubdomains2D()`
 @*/
 PetscErrorCode PCGASMSetUseDMSubdomains(PC pc, PetscBool flg)
@@ -1904,7 +1899,7 @@ PetscErrorCode PCGASMSetUseDMSubdomains(PC pc, PetscBool flg)
 
   Level: intermediate
 
-.seealso: `PCGASM`, `PCGASMSetUseDMSubdomains()`, `PCGASMSetOverlap()`
+.seealso: [](ch_ksp), `PCGASM`, `PCGASMSetUseDMSubdomains()`, `PCGASMSetOverlap()`
           `PCGASMCreateSubdomains2D()`
 @*/
 PetscErrorCode PCGASMGetUseDMSubdomains(PC pc, PetscBool *flg)
