@@ -18,7 +18,7 @@ extern PetscErrorCode MatSolverTypeDestroy(void);
 static PetscBool      MatPackageInitialized = PETSC_FALSE;
 
 /*@C
-  MatFinalizePackage - This function destroys everything in the Petsc interface to the `Mat`
+  MatFinalizePackage - This function destroys everything in the PETSc interface to the `Mat`
   package. It is called from `PetscFinalize()`.
 
   Level: developer
@@ -66,55 +66,58 @@ PETSC_INTERN PetscErrorCode MatSolverTypeRegister_DENSEHIP(void);
 #endif
 
 #if defined(PETSC_HAVE_MUMPS)
-PETSC_EXTERN PetscErrorCode MatSolverTypeRegister_MUMPS(void);
+PETSC_INTERN PetscErrorCode MatSolverTypeRegister_MUMPS(void);
 #endif
 #if defined(PETSC_HAVE_CUDA)
-PETSC_EXTERN PetscErrorCode MatSolverTypeRegister_CUSPARSE(void);
+PETSC_INTERN PetscErrorCode MatSolverTypeRegister_CUSPARSE(void);
 #endif
 #if defined(PETSC_HAVE_HIP)
-PETSC_EXTERN PetscErrorCode MatSolverTypeRegister_HIPSPARSE(void);
+PETSC_INTERN PetscErrorCode MatSolverTypeRegister_HIPSPARSE(void);
 #endif
 #if defined(PETSC_HAVE_KOKKOS_KERNELS)
-PETSC_EXTERN PetscErrorCode MatSolverTypeRegister_KOKKOS(void);
+PETSC_INTERN PetscErrorCode MatSolverTypeRegister_Kokkos(void);
 #endif
 #if defined(PETSC_HAVE_VIENNACL)
-PETSC_EXTERN PetscErrorCode MatSolverTypeRegister_ViennaCL(void);
+PETSC_INTERN PetscErrorCode MatSolverTypeRegister_ViennaCL(void);
 #endif
 #if defined(PETSC_HAVE_ELEMENTAL)
-PETSC_EXTERN PetscErrorCode MatSolverTypeRegister_Elemental(void);
+PETSC_INTERN PetscErrorCode MatSolverTypeRegister_Elemental(void);
 #endif
 #if defined(PETSC_HAVE_SCALAPACK)
-PETSC_EXTERN PetscErrorCode MatSolverTypeRegister_ScaLAPACK(void);
+PETSC_INTERN PetscErrorCode MatSolverTypeRegister_ScaLAPACK(void);
 #endif
 #if defined(PETSC_HAVE_MATLAB)
-PETSC_EXTERN PetscErrorCode MatSolverTypeRegister_Matlab(void);
+PETSC_INTERN PetscErrorCode MatSolverTypeRegister_Matlab(void);
 #endif
 #if defined(PETSC_HAVE_ESSL)
-PETSC_EXTERN PetscErrorCode MatSolverTypeRegister_Essl(void);
+PETSC_INTERN PetscErrorCode MatSolverTypeRegister_Essl(void);
 #endif
 #if defined(PETSC_HAVE_SUPERLU)
-PETSC_EXTERN PetscErrorCode MatSolverTypeRegister_SuperLU(void);
+PETSC_INTERN PetscErrorCode MatSolverTypeRegister_SuperLU(void);
 #endif
 #if defined(PETSC_HAVE_STRUMPACK)
-PETSC_EXTERN PetscErrorCode MatSolverTypeRegister_STRUMPACK(void);
+PETSC_INTERN PetscErrorCode MatSolverTypeRegister_STRUMPACK(void);
 #endif
 #if defined(PETSC_HAVE_PASTIX)
-PETSC_EXTERN PetscErrorCode MatSolverTypeRegister_Pastix(void);
+PETSC_INTERN PetscErrorCode MatSolverTypeRegister_Pastix(void);
 #endif
 #if defined(PETSC_HAVE_SUPERLU_DIST)
-PETSC_EXTERN PetscErrorCode MatSolverTypeRegister_SuperLU_DIST(void);
+PETSC_INTERN PetscErrorCode MatSolverTypeRegister_SuperLU_DIST(void);
 #endif
 #if defined(PETSC_HAVE_MKL_PARDISO)
-PETSC_EXTERN PetscErrorCode MatSolverTypeRegister_MKL_Pardiso(void);
+PETSC_INTERN PetscErrorCode MatSolverTypeRegister_MKL_Pardiso(void);
 #endif
 #if defined(PETSC_HAVE_MKL_CPARDISO)
-PETSC_EXTERN PetscErrorCode MatSolverTypeRegister_MKL_CPardiso(void);
+PETSC_INTERN PetscErrorCode MatSolverTypeRegister_MKL_CPardiso(void);
 #endif
 #if defined(PETSC_HAVE_SUITESPARSE)
-PETSC_EXTERN PetscErrorCode MatSolverTypeRegister_SuiteSparse(void);
+PETSC_INTERN PetscErrorCode MatSolverTypeRegister_SuiteSparse(void);
 #endif
 #if defined(PETSC_HAVE_LUSOL)
-PETSC_EXTERN PetscErrorCode MatSolverTypeRegister_Lusol(void);
+PETSC_INTERN PetscErrorCode MatSolverTypeRegister_Lusol(void);
+#endif
+#if defined(PETSC_HAVE_HTOOL)
+PETSC_INTERN PetscErrorCode MatSolverTypeRegister_Htool(void);
 #endif
 
 PETSC_INTERN PetscErrorCode MatGetFactor_seqaij_petsc(Mat, MatFactorType, Mat *);
@@ -140,6 +143,9 @@ PETSC_INTERN PetscErrorCode PetscBenchCreate_HPL(PetscBench);
 
   Level: developer
 
+  Note:
+  This function never needs to be called by PETSc users.
+
 .seealso: [](ch_matrices), `Mat`, `PetscInitialize()`, `MatFinalizePackage()`
 @*/
 PetscErrorCode MatInitializePackage(void)
@@ -265,6 +271,7 @@ PetscErrorCode MatInitializePackage(void)
   PetscCall(PetscLogEventRegister("MatDenseCopyTo", MAT_CLASSID, &MAT_DenseCopyToGPU));
   PetscCall(PetscLogEventRegister("MatDenseCopyFrom", MAT_CLASSID, &MAT_DenseCopyFromGPU));
   PetscCall(PetscLogEventRegister("MatSetValBatch", MAT_CLASSID, &MAT_SetValuesBatch));
+  PetscCall(PetscLogEventRegister("MatCreateGraph", MAT_CLASSID, &MAT_CreateGraph));
 
   PetscCall(PetscLogEventRegister("MatColoringApply", MAT_COLORING_CLASSID, &MATCOLORING_Apply));
   PetscCall(PetscLogEventRegister("MatColoringComm", MAT_COLORING_CLASSID, &MATCOLORING_Comm));
@@ -386,7 +393,7 @@ PetscErrorCode MatInitializePackage(void)
   PetscCall(MatSolverTypeRegister_HIPSPARSE());
 #endif
 #if defined(PETSC_HAVE_KOKKOS_KERNELS)
-  PetscCall(MatSolverTypeRegister_KOKKOS());
+  PetscCall(MatSolverTypeRegister_Kokkos());
 #endif
 #if defined(PETSC_HAVE_VIENNACL)
   PetscCall(MatSolverTypeRegister_ViennaCL());
@@ -427,6 +434,9 @@ PetscErrorCode MatInitializePackage(void)
 #if defined(PETSC_HAVE_LUSOL)
   PetscCall(MatSolverTypeRegister_Lusol());
 #endif
+#if defined(PETSC_HAVE_HTOOL)
+  PetscCall(MatSolverTypeRegister_Htool());
+#endif
 #if defined(PETSC_HAVE_HPL)
   PetscCall(PetscBenchRegister(PETSCBMHPL, PetscBenchCreate_HPL));
 #endif
