@@ -23,7 +23,7 @@ static PetscErrorCode PetscViewerFileClose_ASCII(PetscViewer viewer)
       PetscCall(PetscStrlcat(par, vascii->filename, sizeof(par)));
 #if defined(PETSC_HAVE_POPEN)
       PetscCall(PetscPOpen(PETSC_COMM_SELF, NULL, par, "r", &fp));
-      PetscCheck(!fgets(buf, 1024, fp), PETSC_COMM_SELF, PETSC_ERR_LIB, "Error from compression command %s\n%s", par, buf);
+      PetscCheck(!fgets(buf, 1024, fp), PETSC_COMM_SELF, PETSC_ERR_LIB, "Error from compression command %s %s", par, buf);
       PetscCall(PetscPClose(PETSC_COMM_SELF, fp));
 #else
       SETERRQ(PETSC_COMM_SELF, PETSC_ERR_SUP_SYS, "Cannot run external programs on this machine");
@@ -34,7 +34,6 @@ static PetscErrorCode PetscViewerFileClose_ASCII(PetscViewer viewer)
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-/* ----------------------------------------------------------------------*/
 static PetscErrorCode PetscViewerDestroy_ASCII(PetscViewer viewer)
 {
   PetscViewer_ASCII *vascii = (PetscViewer_ASCII *)viewer->data;
@@ -47,7 +46,7 @@ static PetscErrorCode PetscViewerDestroy_ASCII(PetscViewer viewer)
   PetscCall(PetscFree(vascii));
 
   /* remove the viewer from the list in the MPI Communicator */
-  if (Petsc_Viewer_keyval == MPI_KEYVAL_INVALID) PetscCallMPI(MPI_Comm_create_keyval(MPI_COMM_NULL_COPY_FN, Petsc_DelViewer, &Petsc_Viewer_keyval, (void *)0));
+  if (Petsc_Viewer_keyval == MPI_KEYVAL_INVALID) PetscCallMPI(MPI_Comm_create_keyval(MPI_COMM_NULL_COPY_FN, Petsc_DelViewer, &Petsc_Viewer_keyval, NULL));
 
   PetscCallMPI(MPI_Comm_get_attr(PetscObjectComm((PetscObject)viewer), Petsc_Viewer_keyval, (void **)&vlink, (PetscMPIInt *)&flg));
   if (flg) {
@@ -96,84 +95,6 @@ static PetscErrorCode PetscViewerDestroy_ASCII_SubViewer(PetscViewer viewer)
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-static PetscErrorCode PetscViewerFlush_ASCII(PetscViewer viewer)
-{
-  PetscViewer_ASCII *vascii = (PetscViewer_ASCII *)viewer->data;
-  MPI_Comm           comm;
-  PetscMPIInt        rank, size;
-  FILE              *fd = vascii->fd;
-
-  PetscFunctionBegin;
-  PetscCheck(!vascii->sviewer, PetscObjectComm((PetscObject)viewer), PETSC_ERR_ARG_WRONGSTATE, "Cannot call with outstanding call to PetscViewerRestoreSubViewer()");
-  PetscCall(PetscObjectGetComm((PetscObject)viewer, &comm));
-  PetscCallMPI(MPI_Comm_rank(comm, &rank));
-  PetscCallMPI(MPI_Comm_size(comm, &size));
-
-  if (!vascii->bviewer && rank == 0 && (vascii->mode != FILE_MODE_READ)) PetscCall(PetscFFlush(vascii->fd));
-
-  if (vascii->allowsynchronized) {
-    PetscMPIInt tag, i, j, n = 0, dummy = 0;
-    char       *message;
-    MPI_Status  status;
-
-    PetscCall(PetscCommDuplicate(comm, &comm, &tag));
-
-    /* First processor waits for messages from all other processors */
-    if (rank == 0) {
-      /* flush my own messages that I may have queued up */
-      PrintfQueue next = vascii->petsc_printfqueuebase, previous;
-      for (i = 0; i < vascii->petsc_printfqueuelength; i++) {
-        if (!vascii->bviewer) {
-          PetscCall(PetscFPrintf(comm, fd, "%s", next->string));
-        } else {
-          PetscCall(PetscViewerASCIISynchronizedPrintf(vascii->bviewer, "%s", next->string));
-        }
-        previous = next;
-        next     = next->next;
-        PetscCall(PetscFree(previous->string));
-        PetscCall(PetscFree(previous));
-      }
-      vascii->petsc_printfqueue       = NULL;
-      vascii->petsc_printfqueuelength = 0;
-      for (i = 1; i < size; i++) {
-        /* to prevent a flood of messages to process zero, request each message separately */
-        PetscCallMPI(MPI_Send(&dummy, 1, MPI_INT, i, tag, comm));
-        PetscCallMPI(MPI_Recv(&n, 1, MPI_INT, i, tag, comm, &status));
-        for (j = 0; j < n; j++) {
-          PetscMPIInt size = 0;
-
-          PetscCallMPI(MPI_Recv(&size, 1, MPI_INT, i, tag, comm, &status));
-          PetscCall(PetscMalloc1(size, &message));
-          PetscCallMPI(MPI_Recv(message, size, MPI_CHAR, i, tag, comm, &status));
-          if (!vascii->bviewer) {
-            PetscCall(PetscFPrintf(comm, fd, "%s", message));
-          } else {
-            PetscCall(PetscViewerASCIISynchronizedPrintf(vascii->bviewer, "%s", message));
-          }
-          PetscCall(PetscFree(message));
-        }
-      }
-    } else { /* other processors send queue to processor 0 */
-      PrintfQueue next = vascii->petsc_printfqueuebase, previous;
-
-      PetscCallMPI(MPI_Recv(&dummy, 1, MPI_INT, 0, tag, comm, &status));
-      PetscCallMPI(MPI_Send(&vascii->petsc_printfqueuelength, 1, MPI_INT, 0, tag, comm));
-      for (i = 0; i < vascii->petsc_printfqueuelength; i++) {
-        PetscCallMPI(MPI_Send(&next->size, 1, MPI_INT, 0, tag, comm));
-        PetscCallMPI(MPI_Send(next->string, next->size, MPI_CHAR, 0, tag, comm));
-        previous = next;
-        next     = next->next;
-        PetscCall(PetscFree(previous->string));
-        PetscCall(PetscFree(previous));
-      }
-      vascii->petsc_printfqueue       = NULL;
-      vascii->petsc_printfqueuelength = 0;
-    }
-    PetscCall(PetscCommDestroy(&comm));
-  }
-  PetscFunctionReturn(PETSC_SUCCESS);
-}
-
 /*@C
   PetscViewerASCIIGetPointer - Extracts the file pointer from an ASCII `PetscViewer`.
 
@@ -198,6 +119,7 @@ PetscErrorCode PetscViewerASCIIGetPointer(PetscViewer viewer, FILE **fd)
   PetscViewer_ASCII *vascii = (PetscViewer_ASCII *)viewer->data;
 
   PetscFunctionBegin;
+  PetscCheck(!vascii->fileunit, PetscObjectComm((PetscObject)viewer), PETSC_ERR_ARG_WRONGSTATE, "Cannot request file pointer for viewers that use Fortran files");
   *fd = vascii->fd;
   PetscFunctionReturn(PETSC_SUCCESS);
 }
@@ -349,7 +271,7 @@ PetscErrorCode PetscViewerASCIISubtractTab(PetscViewer viewer, PetscInt tabs)
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-/*@C
+/*@
   PetscViewerASCIIPushSynchronized - Allows calls to `PetscViewerASCIISynchronizedPrintf()` for this viewer
 
   Collective
@@ -379,7 +301,7 @@ PetscErrorCode PetscViewerASCIIPushSynchronized(PetscViewer viewer)
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-/*@C
+/*@
   PetscViewerASCIIPopSynchronized - Undoes most recent `PetscViewerASCIIPushSynchronized()` for this viewer
 
   Collective
@@ -412,7 +334,7 @@ PetscErrorCode PetscViewerASCIIPopSynchronized(PetscViewer viewer)
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-/*@C
+/*@
   PetscViewerASCIIPushTab - Adds one more tab to the amount that `PetscViewerASCIIPrintf()`
   lines are tabbed.
 
@@ -439,7 +361,7 @@ PetscErrorCode PetscViewerASCIIPushTab(PetscViewer viewer)
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-/*@C
+/*@
   PetscViewerASCIIPopTab - Removes one tab from the amount that `PetscViewerASCIIPrintf()` lines are tabbed that was provided by
   `PetscViewerASCIIPushTab()`
 
@@ -505,26 +427,75 @@ PetscErrorCode PetscViewerASCIIUseTabs(PetscViewer viewer, PetscBool flg)
 #if defined(PETSC_USE_FORTRAN_BINDINGS)
 
   #if defined(PETSC_HAVE_FORTRAN_CAPS)
-    #define petscviewerasciiopenwithfileunit_ PETSCVIEWERASCIIOPENWITHFILEUNIT
-    #define petscviewerasciisetfilefileunit_  PETSCVIEWERASCIISETFILEUNIT
-    #define petscfortranprinttounit_          PETSCFORTRANPRINTTOUNIT
+    #define petscviewerasciiopenwithfileunit_  PETSCVIEWERASCIIOPENWITHFILEUNIT
+    #define petscviewerasciisetfileunit_       PETSCVIEWERASCIISETFILEUNIT
+    #define petscviewerasciistdoutsetfileunit_ PETSCVIEWERASCIISTDOUTSETFILEUNIT
+    #define petscfortranprinttofileunit_       PETSCFORTRANPRINTTOFILEUNIT
   #elif !defined(PETSC_HAVE_FORTRAN_UNDERSCORE)
-    #define petscviewerasciiopenwithfileunit_ petscviewerasciiopenwithfileunit
-    #define petscviewerasciisetfileunit_      petscviewerasciisetfileunit
-    #define petscfortranprinttounit_          petscfortranprinttounit
+    #define petscviewerasciiopenwithfileunit_  petscviewerasciiopenwithfileunit
+    #define petscviewerasciisetfileunit_       petscviewerasciisetfileunit
+    #define petscviewerasciistdoutsetfileunit_ petscviewerasciistdoutsetfileunit
+    #define petscfortranprinttofileunit_       petscfortranprinttofileunit
   #endif
 
   #if defined(__cplusplus)
-extern "C" void petscfortranprinttounit_(PetscInt *, const char *, PetscErrorCode *, PETSC_FORTRAN_CHARLEN_T);
+extern "C" void petscfortranprinttofileunit_(int *, const char *, PetscErrorCode *, PETSC_FORTRAN_CHARLEN_T);
   #else
-extern void petscfortranprinttounit_(PetscInt *, const char *, PetscErrorCode *, PETSC_FORTRAN_CHARLEN_T);
+extern void petscfortranprinttofileunit_(int *, const char *, PetscErrorCode *, PETSC_FORTRAN_CHARLEN_T);
   #endif
 
   #define PETSCDEFAULTBUFFERSIZE 8 * 1024
 
+static int PETSC_VIEWER_ASCII_STDOUT_fileunit = 0;
+
 // PetscClangLinter pragma disable: -fdoc-synopsis-macro-explicit-synopsis-valid-header
 /*MC
-  PetscViewerASCIISetFileUnit - sets the `PETSCVIEWERASCII` to write to a Fortran IO unit
+  PetscViewerASCIIStdoutSetFileUnit - sets `PETSC_VIEWER_STDOUT_()` to write to a Fortran IO unit
+
+  Synopsis:
+  #include <petscviewer.h>
+  void PetscViewerASCIIStdoutSetFileUnit(PetscInt unit, PetscErrorCode ierr)
+
+  Input Parameter:
+. unit - the unit number
+
+  Output Parameter:
+. ierr - the error code
+
+  Level: intermediate
+
+  Notes:
+  Can be called before `PetscInitialize()`
+
+  Immediately changes the output for all `PETSC_VIEWER_STDOUT_()` viewers
+
+  This may not work currently with some viewers that (improperly) use the `fd` directly instead of `PetscViewerASCIIPrintf()`
+
+  With this option, for example, `-log_options` results will be saved to the Fortran file
+
+  Any process may call this but only the unit passed on the first process is used
+
+  Fortran Note:
+  Only for Fortran
+
+  Developer Note:
+  `PetscViewerASCIIWORLDSetFilename()` and `PetscViewerASCIIWORLDSetFILE()` could be added
+
+.seealso: `PetscViewerASCIISetFILE()`, `PETSCVIEWERASCII`, `PetscViewerASCIIOpenWithFileUnit()`, `PetscViewerASCIIStdoutSetFileUnit()`,
+          `PETSC_VIEWER_STDOUT_()`, `PetscViewerASCIIGetStdout()`
+M*/
+PETSC_EXTERN void petscviewerasciistdoutsetfileunit_(int *unit, PetscErrorCode *ierr)
+{
+  #if defined(PETSC_USE_FORTRAN_BINDINGS)
+  PETSC_VIEWER_ASCII_STDOUT_fileunit = *unit;
+  #endif
+}
+
+  #include <petsc/private/ftnimpl.h>
+
+// PetscClangLinter pragma disable: -fdoc-synopsis-macro-explicit-synopsis-valid-header
+/*MC
+  PetscViewerASCIISetFileUnit - sets the `PETSCVIEWERASCII` `PetscViewer` to write to a Fortran IO unit
 
   Synopsis:
   #include <petscviewer.h>
@@ -545,12 +516,15 @@ extern void petscfortranprinttounit_(PetscInt *, const char *, PetscErrorCode *,
   Fortran Notes:
   Only for Fortran, use  `PetscViewerASCIISetFILE()` for C
 
-.seealso: `PetscViewerASCIISetFILE()`, `PETSCVIEWERASCII`, `PetscViewerASCIIOpenWithFileUnit()`
+.seealso: `PetscViewerASCIISetFILE()`, `PETSCVIEWERASCII`, `PetscViewerASCIIOpenWithFileUnit()`, `PetscViewerASCIIStdoutSetFileUnit()`
 M*/
-PETSC_EXTERN void petscviewerasciisetfileunit_(PetscViewer *lab, PetscInt *unit, PetscErrorCode *ierr)
+PETSC_EXTERN void petscviewerasciisetfileunit_(PetscViewer *lab, int *unit, PetscErrorCode *ierr)
 {
-  PetscViewer_ASCII *vascii = (PetscViewer_ASCII *)(*lab)->data;
+  PetscViewer_ASCII *vascii;
+  PetscViewer        v;
 
+  PetscPatchDefaultViewers_Fortran(lab, v);
+  vascii = (PetscViewer_ASCII *)v->data;
   if (vascii->mode == FILE_MODE_READ) {
     *ierr = PETSC_ERR_ARG_WRONGSTATE;
     return;
@@ -564,7 +538,7 @@ PETSC_EXTERN void petscviewerasciisetfileunit_(PetscViewer *lab, PetscInt *unit,
 
   Synopsis:
   #include <petscviewer.h>
-  void PetscViewerASCIIOpenWithFileUnit(MPI_Comm comm, PetscInt unit, PetscViewer viewer, PetscErrorCode ierr)
+  void PetscViewerASCIIOpenWithFileUnit((MPI_Fint comm, integer unit, PetscViewer viewer, PetscErrorCode ierr)
 
   Input Parameters:
 + comm - the `MPI_Comm` to share the viewer
@@ -584,7 +558,7 @@ PETSC_EXTERN void petscviewerasciisetfileunit_(PetscViewer *lab, PetscInt *unit,
 
 .seealso: `PetscViewerASCIISetFileUnit()`, `PetscViewerASCIISetFILE()`, `PETSCVIEWERASCII`, `PetscViewerASCIIOpenWithFILE()`
 M*/
-PETSC_EXTERN void petscviewerasciiopenwithfileunit_(MPI_Comm *comm, PetscInt *unit, PetscViewer *lab, PetscErrorCode *ierr)
+PETSC_EXTERN void petscviewerasciiopenwithfileunit_(MPI_Fint *comm, int *unit, PetscViewer *lab, PetscErrorCode *ierr)
 {
   *ierr = PetscViewerCreate(MPI_Comm_f2c(*(MPI_Fint *)&*comm), lab);
   if (*ierr) return;
@@ -595,7 +569,7 @@ PETSC_EXTERN void petscviewerasciiopenwithfileunit_(MPI_Comm *comm, PetscInt *un
   petscviewerasciisetfileunit_(lab, unit, ierr);
 }
 
-static PetscErrorCode PetscVFPrintfFortran(PetscInt unit, const char format[], va_list Argp)
+static PetscErrorCode PetscVFPrintfFortran(int unit, const char format[], va_list Argp)
 {
   PetscErrorCode ierr;
   char           str[PETSCDEFAULTBUFFERSIZE];
@@ -604,37 +578,97 @@ static PetscErrorCode PetscVFPrintfFortran(PetscInt unit, const char format[], v
   PetscFunctionBegin;
   PetscCall(PetscVSNPrintf(str, sizeof(str), format, NULL, Argp));
   PetscCall(PetscStrlen(str, &len));
-  petscfortranprinttounit_(&unit, str, &ierr, (int)len);
+  petscfortranprinttofileunit_(&unit, str, &ierr, (int)len);
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-static PetscErrorCode PetscFPrintfFortran(PetscInt unit, const char str[])
+static PetscErrorCode PetscFPrintfFortran(int unit, const char str[])
 {
   PetscErrorCode ierr;
   size_t         len;
 
   PetscFunctionBegin;
   PetscCall(PetscStrlen(str, &len));
-  petscfortranprinttounit_(&unit, str, &ierr, (int)len);
+  petscfortranprinttofileunit_(&unit, str, &ierr, (int)len);
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 #else
 
 /* these will never be used; but are needed to link with */
-static PetscErrorCode PetscVFPrintfFortran(PetscInt unit, const char format[], va_list Argp)
+static PetscErrorCode PetscVFPrintfFortran(int unit, const char format[], va_list Argp)
 {
   PetscFunctionBegin;
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-static PetscErrorCode PetscFPrintfFortran(PetscInt unit, const char str[])
+static PetscErrorCode PetscFPrintfFortran(int unit, const char str[])
 {
   PetscFunctionBegin;
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 #endif
 
+/*@
+  PetscViewerASCIIGetStdout - Creates a `PETSCVIEWERASCII` `PetscViewer` shared by all processes
+  in a communicator that prints to `stdout`. Error returning version of `PETSC_VIEWER_STDOUT_()`
+
+  Collective
+
+  Input Parameter:
+. comm - the MPI communicator to share the `PetscViewer`
+
+  Output Parameter:
+. viewer - the viewer
+
+  Level: beginner
+
+  Note:
+  Use `PetscViewerDestroy()` to destroy it
+
+  Developer Note:
+  This should be used in all PETSc source code instead of `PETSC_VIEWER_STDOUT_()` since it allows error checking
+
+.seealso: [](sec_viewers), `PetscViewerASCIIGetStderr()`, `PETSC_VIEWER_DRAW_()`, `PetscViewerASCIIOpen()`, `PETSC_VIEWER_STDERR_`, `PETSC_VIEWER_STDOUT_WORLD`,
+          `PETSC_VIEWER_STDOUT_SELF`
+@*/
+PetscErrorCode PetscViewerASCIIGetStdout(MPI_Comm comm, PetscViewer *viewer)
+{
+  PetscBool flg;
+  MPI_Comm  ncomm;
+
+  PetscFunctionBegin;
+  PetscAssertPointer(viewer, 2);
+  PetscCall(PetscSpinlockLock(&PetscViewerASCIISpinLockStdout));
+  PetscCall(PetscCommDuplicate(comm, &ncomm, NULL));
+  if (Petsc_Viewer_Stdout_keyval == MPI_KEYVAL_INVALID) PetscCallMPI(MPI_Comm_create_keyval(MPI_COMM_NULL_COPY_FN, MPI_COMM_NULL_DELETE_FN, &Petsc_Viewer_Stdout_keyval, NULL));
+  PetscCallMPI(MPI_Comm_get_attr(ncomm, Petsc_Viewer_Stdout_keyval, (void **)viewer, (PetscMPIInt *)&flg));
+  if (!flg) { /* PetscViewer not yet created */
+#if defined(PETSC_USE_FORTRAN_BINDINGS)
+    PetscCallMPI(MPI_Bcast(&PETSC_VIEWER_ASCII_STDOUT_fileunit, 1, MPI_INT, 0, comm));
+    if (PETSC_VIEWER_ASCII_STDOUT_fileunit) {
+      PetscErrorCode ierr;
+      MPI_Fint       fcomm = MPI_Comm_c2f(ncomm);
+
+      petscviewerasciiopenwithfileunit_(&fcomm, &PETSC_VIEWER_ASCII_STDOUT_fileunit, viewer, &ierr);
+    } else
+#endif
+    {
+      PetscCall(PetscViewerCreate(ncomm, viewer));
+      PetscCall(PetscViewerSetType(*viewer, PETSCVIEWERASCII));
+      PetscCall(PetscViewerFileSetName(*viewer, "stdout"));
+    }
+    PetscCall(PetscObjectRegisterDestroy((PetscObject)*viewer));
+    PetscCallMPI(MPI_Comm_set_attr(ncomm, Petsc_Viewer_Stdout_keyval, (void *)*viewer));
+  }
+  PetscCall(PetscCommDestroy(&ncomm));
+  PetscCall(PetscSpinlockUnlock(&PetscViewerASCIISpinLockStdout));
+#if defined(PETSC_USE_FORTRAN_BINDINGS)
+  ((PetscViewer_ASCII *)(*viewer)->data)->fileunit = PETSC_VIEWER_ASCII_STDOUT_fileunit;
+#endif
+  PetscFunctionReturn(PETSC_SUCCESS);
+}
+
 /*@C
   PetscViewerASCIIPrintf - Prints to a file, only from the first
   processor in the `PetscViewer` of type `PETSCVIEWERASCII`
@@ -659,7 +693,7 @@ PetscErrorCode PetscViewerASCIIPrintf(PetscViewer viewer, const char format[], .
 {
   PetscViewer_ASCII *ascii = (PetscViewer_ASCII *)viewer->data;
   PetscMPIInt        rank;
-  PetscInt           tab, intab = ascii->tab;
+  PetscInt           tab = 0, intab = ascii->tab;
   FILE              *fd = ascii->fd;
   PetscBool          iascii;
 
@@ -678,27 +712,16 @@ PetscErrorCode PetscViewerASCIIPrintf(PetscViewer viewer, const char format[], .
     size_t  fullLength;
 
     PetscCall(PetscCalloc1(QUEUESTRINGSIZE, &string));
+    for (; tab < ascii->tab; tab++) { string[2 * tab] = string[2 * tab + 1] = ' '; }
     va_start(Argp, format);
-    PetscCall(PetscVSNPrintf(string, QUEUESTRINGSIZE, format, &fullLength, Argp));
+    PetscCall(PetscVSNPrintf(string + 2 * intab, QUEUESTRINGSIZE - 2 * intab, format, &fullLength, Argp));
     va_end(Argp);
-    PetscCall(PetscViewerASCIISynchronizedPrintf(viewer, "%s", string));
+    PetscCall(PetscViewerASCIISynchronizedPrintf(ascii->bviewer, "%s", string));
     PetscCall(PetscFree(string));
   } else { /* write directly to file */
     va_list Argp;
-    /* flush my own messages that I may have queued up */
-    PrintfQueue next = ascii->petsc_printfqueuebase, previous;
-    PetscInt    i;
-    for (i = 0; i < ascii->petsc_printfqueuelength; i++) {
-      if (!ascii->fileunit) PetscCall(PetscFPrintf(PETSC_COMM_SELF, fd, "%s", next->string));
-      else PetscCall(PetscFPrintfFortran(ascii->fileunit, next->string));
-      previous = next;
-      next     = next->next;
-      PetscCall(PetscFree(previous->string));
-      PetscCall(PetscFree(previous));
-    }
-    ascii->petsc_printfqueue       = NULL;
-    ascii->petsc_printfqueuelength = 0;
-    tab                            = intab;
+
+    tab = intab;
     while (tab--) {
       if (!ascii->fileunit) PetscCall(PetscFPrintf(PETSC_COMM_SELF, fd, "  "));
       else PetscCall(PetscFPrintfFortran(ascii->fileunit, "   "));
@@ -713,7 +736,7 @@ PetscErrorCode PetscViewerASCIIPrintf(PetscViewer viewer, const char format[], .
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-/*@C
+/*@
   PetscViewerFileSetName - Sets the name of the file the `PetscViewer` should use.
 
   Collective
@@ -760,7 +783,7 @@ PetscErrorCode PetscViewerFileSetName(PetscViewer viewer, const char name[])
 
 .seealso: [](sec_viewers), `PetscViewerCreate()`, `PetscViewerSetType()`, `PetscViewerASCIIOpen()`, `PetscViewerBinaryOpen()`, `PetscViewerFileSetName()`
 @*/
-PetscErrorCode PetscViewerFileGetName(PetscViewer viewer, const char **name)
+PetscErrorCode PetscViewerFileGetName(PetscViewer viewer, const char *name[])
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(viewer, PETSC_VIEWER_CLASSID, 1);
@@ -854,8 +877,8 @@ static PetscErrorCode PetscViewerGetSubViewer_ASCII(PetscViewer viewer, MPI_Comm
   PetscViewer_ASCII *vascii = (PetscViewer_ASCII *)viewer->data, *ovascii;
 
   PetscFunctionBegin;
-  PetscCall(PetscViewerASCIIPushSynchronized(viewer));
   PetscCheck(!vascii->sviewer, PETSC_COMM_SELF, PETSC_ERR_ORDER, "SubViewer already obtained from PetscViewer and not restored");
+  PetscCall(PetscViewerASCIIPushSynchronized(viewer));
   /*
      The following line is a bug; it does another PetscViewerASCIIPushSynchronized() on viewer, but if it is removed the code won't work
      because it relies on this behavior in other places. In particular this line causes the synchronized flush to occur when the viewer is destroyed
@@ -865,12 +888,13 @@ static PetscErrorCode PetscViewerGetSubViewer_ASCII(PetscViewer viewer, MPI_Comm
      PCView_GASM().
   */
   PetscCall(PetscViewerASCIIPushSynchronized(viewer));
+  PetscCall(PetscViewerFlush(viewer));
   PetscCall(PetscViewerCreate(subcomm, outviewer));
   PetscCall(PetscViewerSetType(*outviewer, PETSCVIEWERASCII));
   PetscCall(PetscViewerASCIIPushSynchronized(*outviewer));
   ovascii            = (PetscViewer_ASCII *)(*outviewer)->data;
   ovascii->fd        = vascii->fd;
-  ovascii->tab       = vascii->tab;
+  ovascii->fileunit  = vascii->fileunit;
   ovascii->closefile = PETSC_FALSE;
 
   vascii->sviewer                                      = *outviewer;
@@ -892,6 +916,7 @@ static PetscErrorCode PetscViewerRestoreSubViewer_ASCII(PetscViewer viewer, MPI_
   ascii->sviewer             = NULL;
   (*outviewer)->ops->destroy = PetscViewerDestroy_ASCII;
   PetscCall(PetscViewerDestroy(outviewer));
+  PetscCall(PetscViewerFlush(viewer));
   PetscCall(PetscViewerASCIIPopSynchronized(viewer));
   PetscFunctionReturn(PETSC_SUCCESS);
 }
@@ -901,12 +926,93 @@ static PetscErrorCode PetscViewerView_ASCII(PetscViewer v, PetscViewer viewer)
   PetscViewer_ASCII *ascii = (PetscViewer_ASCII *)v->data;
 
   PetscFunctionBegin;
-  if (ascii->filename) PetscCall(PetscViewerASCIIPrintf(viewer, "Filename: %s\n", ascii->filename));
+  if (ascii->fileunit) PetscCall(PetscViewerASCIIPrintf(viewer, "Fortran FILE UNIT: %d\n", ascii->fileunit));
+  else if (ascii->filename) PetscCall(PetscViewerASCIIPrintf(viewer, "Filename: %s\n", ascii->filename));
+  PetscFunctionReturn(PETSC_SUCCESS);
+}
+
+static PetscErrorCode PetscViewerFlush_ASCII(PetscViewer viewer)
+{
+  PetscViewer_ASCII *vascii = (PetscViewer_ASCII *)viewer->data;
+  MPI_Comm           comm;
+  PetscMPIInt        rank, size;
+  FILE              *fd = vascii->fd;
+
+  PetscFunctionBegin;
+  PetscCheck(!vascii->sviewer, PetscObjectComm((PetscObject)viewer), PETSC_ERR_ARG_WRONGSTATE, "Cannot call with outstanding call to PetscViewerRestoreSubViewer()");
+  PetscCall(PetscObjectGetComm((PetscObject)viewer, &comm));
+  PetscCallMPI(MPI_Comm_rank(comm, &rank));
+  PetscCallMPI(MPI_Comm_size(comm, &size));
+
+  if (!vascii->bviewer && rank == 0 && (vascii->mode != FILE_MODE_READ)) PetscCall(PetscFFlush(vascii->fd));
+
+  if (vascii->allowsynchronized) {
+    PetscMPIInt tag, i, j, n = 0, dummy = 0;
+    char       *message;
+    MPI_Status  status;
+
+    PetscCall(PetscCommDuplicate(comm, &comm, &tag));
+
+    /* First processor waits for messages from all other processors */
+    if (rank == 0) {
+      /* flush my own messages that I may have queued up */
+      PrintfQueue next = vascii->petsc_printfqueuebase, previous;
+      for (i = 0; i < vascii->petsc_printfqueuelength; i++) {
+        if (!vascii->bviewer) {
+          if (!vascii->fileunit) PetscCall(PetscFPrintf(comm, fd, "%s", next->string));
+          else PetscCall(PetscFPrintfFortran(vascii->fileunit, next->string));
+        } else {
+          PetscCall(PetscViewerASCIISynchronizedPrintf(vascii->bviewer, "%s", next->string));
+        }
+        previous = next;
+        next     = next->next;
+        PetscCall(PetscFree(previous->string));
+        PetscCall(PetscFree(previous));
+      }
+      vascii->petsc_printfqueue       = NULL;
+      vascii->petsc_printfqueuelength = 0;
+      for (i = 1; i < size; i++) {
+        /* to prevent a flood of messages to process zero, request each message separately */
+        PetscCallMPI(MPI_Send(&dummy, 1, MPI_INT, i, tag, comm));
+        PetscCallMPI(MPI_Recv(&n, 1, MPI_INT, i, tag, comm, &status));
+        for (j = 0; j < n; j++) {
+          size_t size;
+
+          PetscCallMPI(MPI_Recv(&size, 1, MPIU_SIZE_T, i, tag, comm, &status));
+          PetscCall(PetscMalloc1(size, &message));
+          PetscCallMPI(MPI_Recv(message, (PetscMPIInt)size, MPI_CHAR, i, tag, comm, &status));
+          if (!vascii->bviewer) {
+            if (!vascii->fileunit) PetscCall(PetscFPrintf(comm, fd, "%s", message));
+            else PetscCall(PetscFPrintfFortran(vascii->fileunit, message));
+          } else {
+            PetscCall(PetscViewerASCIISynchronizedPrintf(vascii->bviewer, "%s", message));
+          }
+          PetscCall(PetscFree(message));
+        }
+      }
+    } else { /* other processors send queue to processor 0 */
+      PrintfQueue next = vascii->petsc_printfqueuebase, previous;
+
+      PetscCallMPI(MPI_Recv(&dummy, 1, MPI_INT, 0, tag, comm, &status));
+      PetscCallMPI(MPI_Send(&vascii->petsc_printfqueuelength, 1, MPI_INT, 0, tag, comm));
+      for (i = 0; i < vascii->petsc_printfqueuelength; i++) {
+        PetscCallMPI(MPI_Send(&next->size, 1, MPIU_SIZE_T, 0, tag, comm));
+        PetscCallMPI(MPI_Send(next->string, (PetscMPIInt)next->size, MPI_CHAR, 0, tag, comm));
+        previous = next;
+        next     = next->next;
+        PetscCall(PetscFree(previous->string));
+        PetscCall(PetscFree(previous));
+      }
+      vascii->petsc_printfqueue       = NULL;
+      vascii->petsc_printfqueuelength = 0;
+    }
+    PetscCall(PetscCommDestroy(&comm));
+  }
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*MC
-   PETSCVIEWERASCII - A viewer that prints to stdout or an ASCII file
+   PETSCVIEWERASCII - A viewer that prints to `stdout`, `stderr`, or an ASCII file
 
   Level: beginner
 
@@ -979,7 +1085,7 @@ PETSC_EXTERN PetscErrorCode PetscViewerCreate_ASCII(PetscViewer viewer)
     PetscViewerASCIISynchronizedPrintf(viewer, ...);
     ...
     PetscViewerFlush(viewer);
-   PetscViewerASCIIPopSynchronized(viewer);
+    PetscViewerASCIIPopSynchronized(viewer);
 .ve
 
   Fortran Notes:
@@ -993,10 +1099,9 @@ PetscErrorCode PetscViewerASCIISynchronizedPrintf(PetscViewer viewer, const char
 {
   PetscViewer_ASCII *vascii = (PetscViewer_ASCII *)viewer->data;
   PetscMPIInt        rank;
-  PetscInt           tab = vascii->tab;
+  PetscInt           tab = 0;
   MPI_Comm           comm;
-  FILE              *fp;
-  PetscBool          iascii, hasbviewer = PETSC_FALSE;
+  PetscBool          iascii;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(viewer, PETSC_VIEWER_CLASSID, 1);
@@ -1009,35 +1114,30 @@ PetscErrorCode PetscViewerASCIISynchronizedPrintf(PetscViewer viewer, const char
   PetscCallMPI(MPI_Comm_rank(comm, &rank));
 
   if (vascii->bviewer) {
-    hasbviewer = PETSC_TRUE;
-    if (rank == 0) {
-      vascii = (PetscViewer_ASCII *)vascii->bviewer->data;
-      PetscCall(PetscObjectGetComm((PetscObject)viewer, &comm));
-      PetscCallMPI(MPI_Comm_rank(comm, &rank));
-    }
-  }
-
-  fp = vascii->fd;
-
-  if (rank == 0 && !hasbviewer) { /* First processor prints immediately to fp */
+    char   *string;
     va_list Argp;
-    /* flush my own messages that I may have queued up */
-    PrintfQueue next = vascii->petsc_printfqueuebase, previous;
-    PetscInt    i;
-    for (i = 0; i < vascii->petsc_printfqueuelength; i++) {
-      PetscCall(PetscFPrintf(comm, fp, "%s", next->string));
-      previous = next;
-      next     = next->next;
-      PetscCall(PetscFree(previous->string));
-      PetscCall(PetscFree(previous));
-    }
-    vascii->petsc_printfqueue       = NULL;
-    vascii->petsc_printfqueuelength = 0;
+    size_t  fullLength;
 
-    while (tab--) PetscCall(PetscFPrintf(PETSC_COMM_SELF, fp, "  "));
+    PetscCall(PetscCalloc1(QUEUESTRINGSIZE, &string));
+    for (; tab < vascii->tab; tab++) { string[2 * tab] = string[2 * tab + 1] = ' '; }
+    va_start(Argp, format);
+    PetscCall(PetscVSNPrintf(string + 2 * tab, QUEUESTRINGSIZE - 2 * tab, format, &fullLength, Argp));
+    va_end(Argp);
+    PetscCall(PetscViewerASCIISynchronizedPrintf(vascii->bviewer, "%s", string));
+    PetscCall(PetscFree(string));
+  } else if (rank == 0) { /* First processor prints immediately to fp */
+    va_list Argp;
+    FILE   *fp = vascii->fd;
+
+    tab = vascii->tab;
+    while (tab--) {
+      if (!vascii->fileunit) PetscCall(PetscFPrintf(PETSC_COMM_SELF, fp, "  "));
+      else PetscCall(PetscFPrintfFortran(vascii->fileunit, "   "));
+    }
 
     va_start(Argp, format);
-    PetscCall((*PetscVFPrintf)(fp, format, Argp));
+    if (!vascii->fileunit) PetscCall((*PetscVFPrintf)(fp, format, Argp));
+    else PetscCall(PetscVFPrintfFortran(vascii->fileunit, format, Argp));
     va_end(Argp);
     PetscCall(PetscFFlush(fp));
     if (petsc_history) {
@@ -1046,7 +1146,6 @@ PetscErrorCode PetscViewerASCIISynchronizedPrintf(PetscViewer viewer, const char
       va_end(Argp);
       PetscCall(PetscFFlush(petsc_history));
     }
-    va_end(Argp);
   } else { /* other processors add to queue */
     char       *string;
     va_list     Argp;
@@ -1064,12 +1163,14 @@ PetscErrorCode PetscViewerASCIISynchronizedPrintf(PetscViewer viewer, const char
     next->size = QUEUESTRINGSIZE;
     PetscCall(PetscCalloc1(next->size, &next->string));
     string = next->string;
+
+    tab = vascii->tab;
     tab *= 2;
     while (tab--) *string++ = ' ';
     va_start(Argp, format);
     PetscCall(PetscVSNPrintf(string, next->size - 2 * vascii->tab, format, &fullLength, Argp));
     va_end(Argp);
-    if (fullLength > (size_t)(next->size - 2 * vascii->tab)) {
+    if (fullLength > next->size - 2 * vascii->tab) {
       PetscCall(PetscFree(next->string));
       next->size = fullLength + 2 * vascii->tab;
       PetscCall(PetscCalloc1(next->size, &next->string));
@@ -1123,6 +1224,7 @@ PetscErrorCode PetscViewerASCIIRead(PetscViewer viewer, void *data, PetscInt num
     else if (dtype == PETSC_ENUM) ret = fscanf(fd, "%d", &(((int *)data)[i]));
     else if (dtype == PETSC_INT64) ret = fscanf(fd, "%" PetscInt64_FMT, &(((PetscInt64 *)data)[i]));
     else if (dtype == PETSC_LONG) ret = fscanf(fd, "%ld", &(((long *)data)[i]));
+    else if (dtype == PETSC_COUNT) ret = fscanf(fd, "%" PetscCount_FMT, &(((PetscCount *)data)[i]));
     else if (dtype == PETSC_FLOAT) ret = fscanf(fd, "%f", &(((float *)data)[i]));
     else if (dtype == PETSC_DOUBLE) ret = fscanf(fd, "%lg", &(((double *)data)[i]));
 #if defined(PETSC_USE_REAL___FLOAT128)
